@@ -6,7 +6,7 @@ mkdir /app
 mkdir /build
 
 #Set Qt-5.12
-export QT_BASE_DIR=/opt/qt5.12.9
+export QT_BASE_DIR=/opt/qt5.12.10
 export QTDIR=$QT_BASE_DIR
 export PATH=$QT_BASE_DIR/bin:$PATH
 export LD_LIBRARY_PATH=$QT_BASE_DIR/lib/x86_64-linux-gnu:$QT_BASE_DIR/lib:$LD_LIBRARY_PATH
@@ -37,7 +37,7 @@ cd /build
 mkdir build-client
 cd build-client
 cmake -D CMAKE_INSTALL_PREFIX=/usr \
-    -D NO_SHIBBOLETH=1 \
+    -D BUILD_TESTING=OFF \
     -D BUILD_UPDATER=ON \
     -DMIRALL_VERSION_SUFFIX=PR-$DRONE_PULL_REQUEST \
     -DMIRALL_VERSION_BUILD=$DRONE_BUILD_NUMBER \
@@ -48,9 +48,7 @@ make DESTDIR=/app install
 # Move stuff around
 cd /app
 
-mv ./usr/lib/x86_64-linux-gnu/nextcloud/* ./usr/lib/x86_64-linux-gnu/
 mv ./usr/lib/x86_64-linux-gnu/* ./usr/lib/
-rm -rf ./usr/lib/nextcloud
 rm -rf ./usr/lib/cmake
 rm -rf ./usr/include
 rm -rf ./usr/mkspecs
