@@ -28,6 +28,8 @@
 #include "common/utility.h"
 #include "common/ownsql.h"
 #include "common/syncjournalfilerecord.h"
+#include "common/result.h"
+#include "common/pinstate.h"
 
 namespace OCC {
 class SyncJournalFileRecord;
@@ -61,17 +63,26 @@ public:
     bool getFileRecordByInode(quint64 inode, SyncJournalFileRecord *rec);
     bool getFileRecordsByFileId(const QByteArray &fileId, const std::function<void(const SyncJournalFileRecord &)> &rowCallback);
     bool getFilesBelowPath(const QByteArray &path, const std::function<void(const SyncJournalFileRecord&)> &rowCallback);
+    bool listFilesInPath(const QByteArray &path, const std::function<void(const SyncJournalFileRecord&)> &rowCallback);
     bool setFileRecord(const SyncJournalFileRecord &record);
 
-    /// Like setFileRecord, but preserves checksums
-    bool setFileRecordMetadata(const SyncJournalFileRecord &record);
-
     bool deleteFileRecord(const QString &filename, bool recursively = false);
     bool updateFileRecordChecksum(const QString &filename,
         const QByteArray &contentChecksum,
         const QByteArray &contentChecksumType);
     bool updateLocalMetadata(const QString &filename,
-        qint64 modtime, quint64 size, quint64 inode);
+        qint64 modtime, qint64 size, quint64 inode);
+
+    /// Return value for hasHydratedOrDehydratedFiles()
+    struct HasHydratedDehydrated
+    {
+        bool hasHydrated = false;
+        bool hasDehydrated = false;
+    };
+
+    /** Returns whether the item or any subitems are dehydrated */
+    Optional<HasHydratedDehydrated> hasHydratedOrDehydratedFiles(const QByteArray &filename);
+
     bool exists();
     void walCheckpoint();
 
@@ -95,8 +106,8 @@ public:
     struct UploadInfo
     {
         int _chunk = 0;
-        quint64 _transferid = 0;
-        quint64 _size = 0; //currently unused
+        uint _transferid = 0;
+        qint64 _size = 0;
         qint64 _modtime = 0;
         int _errorCount = 0;
         bool _valid = false;
@@ -111,9 +122,10 @@ public:
 
     struct PollInfo
     {
-        QString _file;
-        QString _url;
-        qint64 _modtime;
+        QString _file; // The relative path of a file
+        QString _url; // the poll url. (This pollinfo is invalid if _url is empty)
+        qint64 _modtime; // The modtime of the file being uploaded
+        qint64 _fileSize;
     };
 
     DownloadInfo getDownloadInfo(const QString &file);
@@ -129,6 +141,9 @@ public:
     SyncJournalErrorBlacklistRecord errorBlacklistEntry(const QString &);
     bool deleteStaleErrorBlacklistEntries(const QSet<QString> &keep);
 
+    /// Delete flags table entries that have no metadata correspondent
+    void deleteStaleFlagsEntries();
+
     void avoidRenamesOnNextSync(const QString &path) { avoidRenamesOnNextSync(path.toUtf8()); }
     void avoidRenamesOnNextSync(const QByteArray &path);
     void setPollInfo(const PollInfo &);
@@ -169,8 +184,8 @@ public:
      * Any setFileRecord() call to affected directories before the next sync run will be
      * adjusted to retain the invalid etag via _etagStorageFilter.
      */
-    void avoidReadFromDbOnNextSync(const QString &fileName) { avoidReadFromDbOnNextSync(fileName.toUtf8()); }
-    void avoidReadFromDbOnNextSync(const QByteArray &fileName);
+    void schedulePathForRemoteDiscovery(const QString &fileName) { schedulePathForRemoteDiscovery(fileName.toUtf8()); }
+    void schedulePathForRemoteDiscovery(const QByteArray &fileName);
 
     /**
      * Wipe _etagStorageFilter. Also done implicitly on close().
@@ -180,25 +195,30 @@ public:
     /**
      * Ensures full remote discovery happens on the next sync.
      *
-     * Equivalent to calling avoidReadFromDbOnNextSync() for all files.
+     * Equivalent to calling schedulePathForRemoteDiscovery() for all files.
      */
     void forceRemoteDiscoveryNextSync();
 
-    bool postSyncCleanup(const QSet<QString> &filepathsToKeep,
-        const QSet<QString> &prefixesToKeep);
-
     /* Because sqlite transactions are really slow, we encapsulate everything in big transactions
      * Commit will actually commit the transaction and create a new one.
      */
     void commit(const QString &context, bool startTrans = true);
     void commitIfNeededAndStartNewTransaction(const QString &context);
 
-    void close();
-
-    /**
-     * return true if everything is correct
+    /** Open the db if it isn't already.
+     *
+     * This usually creates some temporary files next to the db file, like
+     * $dbfile-shm or $dbfile-wal.
+     *
+     * returns true if it could be openend or is currently opened.
      */
-    bool isConnected();
+    bool open();
+
+    /** Returns whether the db is currently openend. */
+    bool isOpen();
+
+    /** Close the database */
+    void close();
 
     /**
      * Returns the checksum type for an id.
@@ -241,6 +261,109 @@ public:
      */
     void clearFileTable();
 
+    /**
+     * Set the 'ItemTypeVirtualFileDownload' to all the files that have the ItemTypeVirtualFile flag
+     * within the directory specified path path
+     *
+     * The path "" marks everything.
+     */
+    void markVirtualFileForDownloadRecursively(const QByteArray &path);
+
+    /** Grouping for all functions relating to pin states,
+     *
+     * Use internalPinStates() to get at them.
+     */
+    struct OCSYNC_EXPORT PinStateInterface
+    {
+        PinStateInterface(const PinStateInterface &) = delete;
+        PinStateInterface(PinStateInterface &&) = delete;
+
+        /**
+         * Gets the PinState for the path without considering parents.
+         *
+         * If a path has no explicit PinState "Inherited" is returned.
+         *
+         * The path should not have a trailing slash.
+         * It's valid to use the root path "".
+         *
+         * Returns none on db error.
+         */
+        Optional<PinState> rawForPath(const QByteArray &path);
+
+        /**
+         * Gets the PinState for the path after inheriting from parents.
+         *
+         * If the exact path has no entry or has an Inherited state,
+         * the state of the closest parent path is returned.
+         *
+         * The path should not have a trailing slash.
+         * It's valid to use the root path "".
+         *
+         * Never returns PinState::Inherited. If the root is "Inherited"
+         * or there's an error, "AlwaysLocal" is returned.
+         *
+         * Returns none on db error.
+         */
+        Optional<PinState> effectiveForPath(const QByteArray &path);
+
+        /**
+         * Like effectiveForPath() but also considers subitem pin states.
+         *
+         * If the path's pin state and all subitem's pin states are identical
+         * then that pin state will be returned.
+         *
+         * If some subitem's pin state is different from the path's state,
+         * PinState::Inherited will be returned. Inherited isn't returned in
+         * any other cases.
+         *
+         * It's valid to use the root path "".
+         * Returns none on db error.
+         */
+        Optional<PinState> effectiveForPathRecursive(const QByteArray &path);
+
+        /**
+         * Sets a path's pin state.
+         *
+         * The path should not have a trailing slash.
+         * It's valid to use the root path "".
+         */
+        void setForPath(const QByteArray &path, PinState state);
+
+        /**
+         * Wipes pin states for a path and below.
+         *
+         * Used when the user asks a subtree to have a particular pin state.
+         * The path should not have a trailing slash.
+         * The path "" wipes every entry.
+         */
+        void wipeForPathAndBelow(const QByteArray &path);
+
+        /**
+         * Returns list of all paths with their pin state as in the db.
+         *
+         * Returns nothing on db error.
+         * Note that this will have an entry for "".
+         */
+        Optional<QVector<QPair<QByteArray, PinState>>> rawList();
+
+        SyncJournalDb *_db;
+    };
+    friend struct PinStateInterface;
+
+    /** Access to PinStates stored in the database.
+     *
+     * Important: Not all vfs plugins store the pin states in the database,
+     * prefer to use Vfs::pinState() etc.
+     */
+    PinStateInterface internalPinStates();
+
+    /**
+     * Only used for auto-test:
+     * when positive, will decrease the counter for every database operation.
+     * reaching 0 makes the operation fails
+     */
+    int autotestFailCounter = -1;
+
 private:
     int getFileRecordCount();
     bool updateDatabaseStructure();
@@ -264,6 +387,7 @@ private:
     SqlDatabase _db;
     QString _dbFile;
     QMutex _mutex; // Public functions are protected with the mutex.
+    QMap<QByteArray, int> _checksymTypeCache;
     int _transaction;
     bool _metadataTableIsEmpty;
 
@@ -273,6 +397,7 @@ private:
     SqlQuery _getFileRecordQueryByFileId;
     SqlQuery _getFilesBelowPathQuery;
     SqlQuery _getAllFilesQuery;
+    SqlQuery _listFilesInPathQuery;
     SqlQuery _setFileRecordQuery;
     SqlQuery _setFileRecordChecksumQuery;
     SqlQuery _setFileRecordLocalMetadataQuery;
@@ -296,10 +421,16 @@ private:
     SqlQuery _getConflictRecordQuery;
     SqlQuery _setConflictRecordQuery;
     SqlQuery _deleteConflictRecordQuery;
+    SqlQuery _getRawPinStateQuery;
+    SqlQuery _getEffectivePinStateQuery;
+    SqlQuery _getSubPinsQuery;
+    SqlQuery _countDehydratedFilesQuery;
+    SqlQuery _setPinStateQuery;
+    SqlQuery _wipePinStateQuery;
 
     /* Storing etags to these folders, or their parent folders, is filtered out.
      *
-     * When avoidReadFromDbOnNextSync() is called some etags to _invalid_ in the
+     * When schedulePathForRemoteDiscovery() is called some etags to _invalid_ in the
      * database. If this is done during a sync run, a later propagation job might
      * undo that by writing the correct etag to the database instead. This filter
      * will prevent this write and instead guarantee the _invalid_ etag stays in
