@@ -26,21 +26,21 @@
 
 #include "c_lib.h"
 #include "c_private.h"
-#include "c_utf8.h"
 
-#include "csync_private.h"
 #include "csync_exclude.h"
-#include "csync_misc.h"
 
 #include "common/utility.h"
+#include "../version.h"
 
 #include <QString>
 #include <QFileInfo>
+#include <QFile>
+#include <QDir>
 
 
 /** Expands C-like escape sequences (in place)
  */
-static void csync_exclude_expand_escapes(QByteArray &input)
+OCSYNC_EXPORT void csync_exclude_expand_escapes(QByteArray &input)
 {
     size_t o = 0;
     char *line = input.data();
@@ -91,39 +91,38 @@ static const char *win_reserved_words_n[] = { "CLOCK$", "$Recycle.Bin" };
  * @param file_name filename
  * @return true if file is reserved, false otherwise
  */
-bool csync_is_windows_reserved_word(const char *filename)
+OCSYNC_EXPORT bool csync_is_windows_reserved_word(const QStringRef &filename)
 {
-    size_t len_filename = strlen(filename);
+    size_t len_filename = filename.size();
 
     // Drive letters
-    if (len_filename == 2 && filename[1] == ':') {
-        if (filename[0] >= 'a' && filename[0] <= 'z') {
+    if (len_filename == 2 && filename.at(1) == QLatin1Char(':')) {
+        if (filename.at(0) >= QLatin1Char('a') && filename.at(0) <= QLatin1Char('z')) {
             return true;
         }
-        if (filename[0] >= 'A' && filename[0] <= 'Z') {
+        if (filename.at(0) >= QLatin1Char('A') && filename.at(0) <= QLatin1Char('Z')) {
             return true;
         }
     }
 
-    if (len_filename == 3 || (len_filename > 3 && filename[3] == '.')) {
+    if (len_filename == 3 || (len_filename > 3 && filename.at(3) == QLatin1Char('.'))) {
         for (const char *word : win_reserved_words_3) {
-            if (c_strncasecmp(filename, word, 3) == 0) {
+            if (filename.left(3).compare(QLatin1String(word), Qt::CaseInsensitive) == 0) {
                 return true;
             }
         }
     }
 
-    if (len_filename == 4 || (len_filename > 4 && filename[4] == '.')) {
+    if (len_filename == 4 || (len_filename > 4 && filename.at(4) == QLatin1Char('.'))) {
         for (const char *word : win_reserved_words_4) {
-            if (c_strncasecmp(filename, word, 4) == 0) {
+            if (filename.left(4).compare(QLatin1String(word), Qt::CaseInsensitive) == 0) {
                 return true;
             }
         }
     }
 
     for (const char *word : win_reserved_words_n) {
-        size_t len_word = strlen(word);
-        if (len_word == len_filename && c_strncasecmp(filename, word, len_word) == 0) {
+        if (filename.compare(QLatin1String(word), Qt::CaseInsensitive) == 0) {
             return true;
         }
     }
@@ -131,51 +130,34 @@ bool csync_is_windows_reserved_word(const char *filename)
     return false;
 }
 
-static CSYNC_EXCLUDE_TYPE _csync_excluded_common(const char *path, bool excludeConflictFiles)
+static CSYNC_EXCLUDE_TYPE _csync_excluded_common(const QString &path, bool excludeConflictFiles)
 {
-    const char *bname = nullptr;
-    size_t blen = 0;
-    int rc = -1;
-    CSYNC_EXCLUDE_TYPE match = CSYNC_NOT_EXCLUDED;
-
     /* split up the path */
-    bname = strrchr(path, '/');
-    if (bname) {
-        bname += 1; // don't include the /
-    } else {
-        bname = path;
+    QStringRef bname(&path);
+    int lastSlash = path.lastIndexOf(QLatin1Char('/'));
+    if (lastSlash >= 0) {
+        bname = path.midRef(lastSlash + 1);
     }
-    blen = strlen(bname);
 
+    size_t blen = bname.size();
     // 9 = strlen(".sync_.db")
-    if (blen >= 9 && bname[0] == '.') {
-        rc = csync_fnmatch("._sync_*.db*", bname, 0);
-        if (rc == 0) {
-            match = CSYNC_FILE_SILENTLY_EXCLUDED;
-            goto out;
+    if (blen >= 9 && bname.at(0) == QLatin1Char('.')) {
+        if (bname.contains(QLatin1String(".db"))) {
+            if (bname.startsWith(QLatin1String("._sync_"), Qt::CaseInsensitive)  // "._sync_*.db*"
+                || bname.startsWith(QLatin1String(".sync_"), Qt::CaseInsensitive) // ".sync_*.db*"
+                || bname.startsWith(QLatin1String(".csync_journal.db"), Qt::CaseInsensitive)) { // ".csync_journal.db*"
+                return CSYNC_FILE_SILENTLY_EXCLUDED;
+            }
         }
-        rc = csync_fnmatch(".sync_*.db*", bname, 0);
-        if (rc == 0) {
-            match = CSYNC_FILE_SILENTLY_EXCLUDED;
-            goto out;
-        }
-        rc = csync_fnmatch(".csync_journal.db*", bname, 0);
-        if (rc == 0) {
-            match = CSYNC_FILE_SILENTLY_EXCLUDED;
-            goto out;
-        }
-        rc = csync_fnmatch(".owncloudsync.log*", bname, 0);
-        if (rc == 0) {
-            match = CSYNC_FILE_SILENTLY_EXCLUDED;
-            goto out;
+        if (bname.startsWith(QLatin1String(".owncloudsync.log"), Qt::CaseInsensitive)) { // ".owncloudsync.log*"
+            return CSYNC_FILE_SILENTLY_EXCLUDED;
         }
     }
 
     // check the strlen and ignore the file if its name is longer than 254 chars.
     // whenever changing this also check createDownloadTmpFileName
     if (blen > 254) {
-        match = CSYNC_FILE_EXCLUDE_LONG_FILENAME;
-        goto out;
+        return CSYNC_FILE_EXCLUDE_LONG_FILENAME;
     }
 
 #ifdef _WIN32
@@ -184,23 +166,24 @@ static CSYNC_EXCLUDE_TYPE _csync_excluded_common(const char *path, bool excludeC
     // as '.' is a separator that is not stored internally, so let's
     // not allow to sync those to avoid file loss/ambiguities (#416)
     if (blen > 1) {
-        if (bname[blen-1]== ' ') {
-            match = CSYNC_FILE_EXCLUDE_TRAILING_SPACE;
-            goto out;
-        } else if (bname[blen-1]== '.' ) {
-            match = CSYNC_FILE_EXCLUDE_INVALID_CHAR;
-            goto out;
+        if (bname.at(blen - 1) == QLatin1Char(' ')) {
+            return CSYNC_FILE_EXCLUDE_TRAILING_SPACE;
+        } else if (bname.at(blen - 1) == QLatin1Char('.')) {
+            return CSYNC_FILE_EXCLUDE_INVALID_CHAR;
         }
     }
 
     if (csync_is_windows_reserved_word(bname)) {
-      match = CSYNC_FILE_EXCLUDE_INVALID_CHAR;
-      goto out;
+        return CSYNC_FILE_EXCLUDE_INVALID_CHAR;
     }
 
     // Filter out characters not allowed in a filename on windows
-    for (const char *p = path; *p; p++) {
-        switch (*p) {
+    for (auto p : path) {
+        const ushort c = p.unicode();
+        if (c < 32) {
+            return CSYNC_FILE_EXCLUDE_INVALID_CHAR;
+        }
+        switch (c) {
         case '\\':
         case ':':
         case '?':
@@ -209,8 +192,7 @@ static CSYNC_EXCLUDE_TYPE _csync_excluded_common(const char *path, bool excludeC
         case '>':
         case '<':
         case '|':
-            match = CSYNC_FILE_EXCLUDE_INVALID_CHAR;
-            goto out;
+            return CSYNC_FILE_EXCLUDE_INVALID_CHAR;
         default:
             break;
         }
@@ -219,24 +201,18 @@ static CSYNC_EXCLUDE_TYPE _csync_excluded_common(const char *path, bool excludeC
 
     /* We create a Desktop.ini on Windows for the sidebar icon, make sure we don't sync it. */
     if (blen == 11 && path == bname) {
-        rc = csync_fnmatch("Desktop.ini", bname, 0);
-        if (rc == 0) {
-            match = CSYNC_FILE_SILENTLY_EXCLUDED;
-            goto out;
+        if (bname.compare(QLatin1String("Desktop.ini"), Qt::CaseInsensitive) == 0) {
+            return CSYNC_FILE_SILENTLY_EXCLUDED;
         }
     }
 
-    if (excludeConflictFiles && OCC::Utility::isConflictFile(bname)) {
-        match = CSYNC_FILE_EXCLUDE_CONFLICT;
-        goto out;
+    if (excludeConflictFiles && OCC::Utility::isConflictFile(path)) {
+        return CSYNC_FILE_EXCLUDE_CONFLICT;
     }
-
-  out:
-
-    return match;
+    return CSYNC_NOT_EXCLUDED;
 }
 
-static QByteArray leftIncludeLast(const QByteArray & arr, char c)
+static QString leftIncludeLast(const QString &arr, const QChar &c)
 {
     // left up to and including `c`
     return arr.left(arr.lastIndexOf(c, arr.size() - 2) + 1);
@@ -244,10 +220,11 @@ static QByteArray leftIncludeLast(const QByteArray & arr, char c)
 
 using namespace OCC;
 
-ExcludedFiles::ExcludedFiles(QString localPath)
-    : _localPath(std::move(localPath))
+ExcludedFiles::ExcludedFiles(const QString &localPath)
+    : _localPath(localPath)
+    , _clientVersion(MIRALL_VERSION_MAJOR, MIRALL_VERSION_MINOR, MIRALL_VERSION_PATCH)
 {
-    Q_ASSERT(_localPath.endsWith("/"));
+    Q_ASSERT(_localPath.endsWith(QStringLiteral("/")));
     // Windows used to use PathMatchSpec which allows *foo to match abc/deffoo.
     _wildcardsMatchSlash = Utility::isWindows();
 
@@ -256,7 +233,7 @@ ExcludedFiles::ExcludedFiles(QString localPath)
         return;
 
     // Load exclude file from base dir
-    QFileInfo fi(_localPath + ".sync-exclude.lst");
+    QFileInfo fi(_localPath + QStringLiteral(".sync-exclude.lst"));
     if (fi.isReadable())
         addInTreeExcludeFilePath(fi.absoluteFilePath());
 }
@@ -265,12 +242,12 @@ ExcludedFiles::~ExcludedFiles() = default;
 
 void ExcludedFiles::addExcludeFilePath(const QString &path)
 {
-    _excludeFiles[_localPath.toUtf8()].append(path);
+    _excludeFiles[_localPath].append(path);
 }
 
 void ExcludedFiles::addInTreeExcludeFilePath(const QString &path)
 {
-    BasePathByteArray basePath = leftIncludeLast(path.toUtf8(), '/');
+    BasePathString basePath = leftIncludeLast(path, QLatin1Char('/'));
     _excludeFiles[basePath].append(path);
 }
 
@@ -279,19 +256,14 @@ void ExcludedFiles::setExcludeConflictFiles(bool onoff)
     _excludeConflictFiles = onoff;
 }
 
-void ExcludedFiles::addManualExclude(const QByteArray &expr)
+void ExcludedFiles::addManualExclude(const QString &expr)
 {
-    addManualExclude(expr, _localPath.toUtf8());
+    addManualExclude(expr, _localPath);
 }
 
-void ExcludedFiles::addManualExclude(const QByteArray &expr, const QByteArray &basePath)
+void ExcludedFiles::addManualExclude(const QString &expr, const QString &basePath)
 {
-#if defined(Q_OS_WIN)
-    Q_ASSERT(basePath.size() >= 2 && basePath.at(1) == ':');
-#else
-    Q_ASSERT(basePath.startsWith('/'));
-#endif
-    Q_ASSERT(basePath.endsWith('/'));
+    Q_ASSERT(basePath.endsWith(QLatin1Char('/')));
 
     auto key = basePath;
     _manualExcludes[key].append(expr);
@@ -311,19 +283,28 @@ void ExcludedFiles::setWildcardsMatchSlash(bool onoff)
     prepare();
 }
 
-bool ExcludedFiles::loadExcludeFile(const QByteArray & basePath, const QString & file)
+void ExcludedFiles::setClientVersion(ExcludedFiles::Version version)
+{
+    _clientVersion = version;
+}
+
+bool ExcludedFiles::loadExcludeFile(const QString &basePath, const QString & file)
 {
     QFile f(file);
     if (!f.open(QIODevice::ReadOnly))
         return false;
 
-    QList<QByteArray> patterns;
+    QStringList patterns;
     while (!f.atEnd()) {
         QByteArray line = f.readLine().trimmed();
+        if (line.startsWith("#!version")) {
+            if (!versionDirectiveKeepNextLine(line))
+                f.readLine();
+        }
         if (line.isEmpty() || line.startsWith('#'))
             continue;
         csync_exclude_expand_escapes(line);
-        patterns.append(line);
+        patterns.append(QString::fromUtf8(line));
     }
     _allExcludes.insert(basePath, patterns);
 
@@ -363,6 +344,32 @@ bool ExcludedFiles::reloadExcludeFiles()
     return success;
 }
 
+bool ExcludedFiles::versionDirectiveKeepNextLine(const QByteArray &directive) const
+{
+    if (!directive.startsWith("#!version"))
+        return true;
+    QByteArrayList args = directive.split(' ');
+    if (args.size() != 3)
+        return true;
+    QByteArray op = args[1];
+    QByteArrayList argVersions = args[2].split('.');
+    if (argVersions.size() != 3)
+        return true;
+
+    auto argVersion = std::make_tuple(argVersions[0].toInt(), argVersions[1].toInt(), argVersions[2].toInt());
+    if (op == "<=")
+        return _clientVersion <= argVersion;
+    if (op == "<")
+        return _clientVersion < argVersion;
+    if (op == ">")
+        return _clientVersion > argVersion;
+    if (op == ">=")
+        return _clientVersion >= argVersion;
+    if (op == "==")
+        return _clientVersion == argVersion;
+    return true;
+}
+
 bool ExcludedFiles::isExcluded(
     const QString &filePath,
     const QString &basePath,
@@ -380,7 +387,7 @@ bool ExcludedFiles::isExcluded(
         // We do want to be able to sync with a hidden folder as the target.
         while (path.size() > basePath.size()) {
             QFileInfo fi(path);
-            if (fi.fileName() != ".sync-exclude.lst"
+            if (fi.fileName() != QStringLiteral(".sync-exclude.lst")
                 && (fi.isHidden() || fi.fileName().startsWith(QLatin1Char('.')))) {
                 return true;
             }
@@ -401,10 +408,10 @@ bool ExcludedFiles::isExcluded(
         relativePath.chop(1);
     }
 
-    return fullPatternMatch(relativePath.toUtf8(), type) != CSYNC_NOT_EXCLUDED;
+    return fullPatternMatch(relativePath, type) != CSYNC_NOT_EXCLUDED;
 }
 
-CSYNC_EXCLUDE_TYPE ExcludedFiles::traversalPatternMatch(const char *path, ItemType filetype)
+CSYNC_EXCLUDE_TYPE ExcludedFiles::traversalPatternMatch(const QString &path, ItemType filetype)
 {
     auto match = _csync_excluded_common(path, _excludeConflictFiles);
     if (match != CSYNC_NOT_EXCLUDED)
@@ -414,7 +421,7 @@ CSYNC_EXCLUDE_TYPE ExcludedFiles::traversalPatternMatch(const char *path, ItemTy
 
     // Directories are guaranteed to be visited before their files
     if (filetype == ItemTypeDirectory) {
-        const auto basePath = QString(_localPath + path + QLatin1Char('/')).toUtf8();
+        const auto basePath = QString(_localPath + path + QLatin1Char('/'));
         const auto fi = QFileInfo(basePath + QStringLiteral(".sync-exclude.lst"));
 
         if (fi.isReadable()) {
@@ -425,18 +432,15 @@ CSYNC_EXCLUDE_TYPE ExcludedFiles::traversalPatternMatch(const char *path, ItemTy
 
     // Check the bname part of the path to see whether the full
     // regex should be run.
-
-    const char *bname = strrchr(path, '/');
-    if (bname) {
-        bname += 1; // don't include the /
-    } else {
-        bname = path;
+    QStringRef bnameStr(&path);
+    int lastSlash = path.lastIndexOf(QLatin1Char('/'));
+    if (lastSlash >= 0) {
+        bnameStr = path.midRef(lastSlash + 1);
     }
-    QString bnameStr = QString::fromUtf8(bname);
 
-    QByteArray basePath(_localPath.toUtf8() + path);
+    QString basePath(_localPath + path);
     while (basePath.size() > _localPath.size()) {
-        basePath = leftIncludeLast(basePath, '/');
+        basePath = leftIncludeLast(basePath, QLatin1Char('/'));
         QRegularExpressionMatch m;
         if (filetype == ItemTypeDirectory
             && _bnameTraversalRegexDir.contains(basePath)) {
@@ -458,17 +462,16 @@ CSYNC_EXCLUDE_TYPE ExcludedFiles::traversalPatternMatch(const char *path, ItemTy
     }
 
     // third capture: full path matching is triggered
-    QString pathStr = QString::fromUtf8(path);
-    basePath = _localPath.toUtf8() + path;
+    basePath = _localPath + path;
     while (basePath.size() > _localPath.size()) {
-        basePath = leftIncludeLast(basePath, '/');
+        basePath = leftIncludeLast(basePath, QLatin1Char('/'));
         QRegularExpressionMatch m;
         if (filetype == ItemTypeDirectory
             && _fullTraversalRegexDir.contains(basePath)) {
-            m = _fullTraversalRegexDir[basePath].match(pathStr);
+            m = _fullTraversalRegexDir[basePath].match(path);
         } else if (filetype == ItemTypeFile
             && _fullTraversalRegexFile.contains(basePath)) {
-            m = _fullTraversalRegexFile[basePath].match(pathStr);
+            m = _fullTraversalRegexFile[basePath].match(path);
         } else {
             continue;
         }
@@ -484,23 +487,23 @@ CSYNC_EXCLUDE_TYPE ExcludedFiles::traversalPatternMatch(const char *path, ItemTy
     return CSYNC_NOT_EXCLUDED;
 }
 
-CSYNC_EXCLUDE_TYPE ExcludedFiles::fullPatternMatch(const char *path, ItemType filetype) const
+CSYNC_EXCLUDE_TYPE ExcludedFiles::fullPatternMatch(const QString &p, ItemType filetype) const
 {
-    auto match = _csync_excluded_common(path, _excludeConflictFiles);
+    auto match = _csync_excluded_common(p, _excludeConflictFiles);
     if (match != CSYNC_NOT_EXCLUDED)
         return match;
     if (_allExcludes.isEmpty())
         return CSYNC_NOT_EXCLUDED;
 
-    QString p = QString::fromUtf8(path);
     // `path` seems to always be relative to `_localPath`, the tests however have not been
     // written that way... this makes the tests happy for now. TODO Fix the tests at some point
-    if (path[0] == '/')
-        ++path;
+    QString path = p;
+    if (path.startsWith(_localPath))
+        path = path.mid(_localPath.size());
 
-    QByteArray basePath(_localPath.toUtf8() + path);
+    QString basePath(_localPath + path);
     while (basePath.size() > _localPath.size()) {
-        basePath = leftIncludeLast(basePath, '/');
+        basePath = leftIncludeLast(basePath, QLatin1Char('/'));
         QRegularExpressionMatch m;
         if (filetype == ItemTypeDirectory
             && _fullRegexDir.contains(basePath)) {
@@ -524,18 +527,12 @@ CSYNC_EXCLUDE_TYPE ExcludedFiles::fullPatternMatch(const char *path, ItemType fi
     return CSYNC_NOT_EXCLUDED;
 }
 
-auto ExcludedFiles::csyncTraversalMatchFun()
-    -> std::function<CSYNC_EXCLUDE_TYPE(const char *path, ItemType filetype)>
-{
-    return [this](const char *path, ItemType filetype) { return this->traversalPatternMatch(path, filetype); };
-}
-
 /**
  * On linux we used to use fnmatch with FNM_PATHNAME, but the windows function we used
  * didn't have that behavior. wildcardsMatchSlash can be used to control which behavior
  * the resulting regex shall use.
  */
-static QString convertToRegexpSyntax(QString exclude, bool wildcardsMatchSlash)
+QString ExcludedFiles::convertToRegexpSyntax(QString exclude, bool wildcardsMatchSlash)
 {
     // Translate *, ?, [...] to their regex variants.
     // The escape sequences \*, \?, \[. \\ have a special meaning,
@@ -560,17 +557,17 @@ static QString convertToRegexpSyntax(QString exclude, bool wildcardsMatchSlash)
         case '*':
             flush();
             if (wildcardsMatchSlash) {
-                regex.append(".*");
+                regex.append(QLatin1String(".*"));
             } else {
-                regex.append("[^/]*");
+                regex.append(QLatin1String("[^/]*"));
             }
             break;
         case '?':
             flush();
             if (wildcardsMatchSlash) {
-                regex.append(".");
+                regex.append(QLatin1Char('.'));
             } else {
-                regex.append("[^/]");
+                regex.append(QStringLiteral("[^/]"));
             }
             break;
         case '[': {
@@ -578,20 +575,20 @@ static QString convertToRegexpSyntax(QString exclude, bool wildcardsMatchSlash)
             // Find the end of the bracket expression
             auto j = i + 1;
             for (; j < len; ++j) {
-                if (exclude[j] == ']')
+                if (exclude[j] == QLatin1Char(']'))
                     break;
-                if (j != len - 1 && exclude[j] == '\\' && exclude[j + 1] == ']')
+                if (j != len - 1 && exclude[j] == QLatin1Char('\\') && exclude[j + 1] == QLatin1Char(']'))
                     ++j;
             }
             if (j == len) {
                 // no matching ], just insert the escaped [
-                regex.append("\\[");
+                regex.append(QStringLiteral("\\["));
                 break;
             }
             // Translate [! to [^
             QString bracketExpr = exclude.mid(i, j - i + 1);
-            if (bracketExpr.startsWith("[!"))
-                bracketExpr[1] = '^';
+            if (bracketExpr.startsWith(QLatin1String("[!")))
+                bracketExpr[1] = QLatin1Char('^');
             regex.append(bracketExpr);
             i = j;
             break;
@@ -599,7 +596,7 @@ static QString convertToRegexpSyntax(QString exclude, bool wildcardsMatchSlash)
         case '\\':
             flush();
             if (i == len - 1) {
-                regex.append("\\\\");
+                regex.append(QStringLiteral("\\\\"));
                 break;
             }
             // '\*' -> '\*', but '\z' -> '\\z'
@@ -625,11 +622,11 @@ static QString convertToRegexpSyntax(QString exclude, bool wildcardsMatchSlash)
     return regex;
 }
 
-static QString extractBnameTrigger(const QString &exclude, bool wildcardsMatchSlash)
+QString ExcludedFiles::extractBnameTrigger(const QString &exclude, bool wildcardsMatchSlash)
 {
     // We can definitely drop everything to the left of a / - that will never match
     // any bname.
-    QString pattern = exclude.mid(exclude.lastIndexOf('/') + 1);
+    QString pattern = exclude.mid(exclude.lastIndexOf(QLatin1Char('/')) + 1);
 
     // Easy case, nothing else can match a slash, so that's it.
     if (!wildcardsMatchSlash)
@@ -656,7 +653,7 @@ static QString extractBnameTrigger(const QString &exclude, bool wildcardsMatchSl
 
     // And if there was a wildcard, it starts with a *
     if (i >= 0)
-        pattern.prepend('*');
+        pattern.prepend(QLatin1Char('*'));
 
     return pattern;
 }
@@ -676,7 +673,7 @@ void ExcludedFiles::prepare()
         prepare(basePath);
 }
 
-void ExcludedFiles::prepare(const BasePathByteArray & basePath)
+void ExcludedFiles::prepare(const BasePathString & basePath)
 {
     Q_ASSERT(_allExcludes.contains(basePath));
 
@@ -717,25 +714,25 @@ void ExcludedFiles::prepare(const BasePathByteArray & basePath)
     auto regexAppend = [](QString &fileDirPattern, QString &dirPattern, const QString &appendMe, bool dirOnly) {
         QString &pattern = dirOnly ? dirPattern : fileDirPattern;
         if (!pattern.isEmpty())
-            pattern.append("|");
+            pattern.append(QLatin1Char('|'));
         pattern.append(appendMe);
     };
 
     for (auto exclude : _allExcludes.value(basePath)) {
-        if (exclude[0] == '\n')
+        if (exclude[0] == QLatin1Char('\n'))
             continue; // empty line
-        if (exclude[0] == '\r')
+        if (exclude[0] == QLatin1Char('\r'))
             continue; // empty line
 
-        bool matchDirOnly = exclude.endsWith('/');
+        bool matchDirOnly = exclude.endsWith(QLatin1Char('/'));
         if (matchDirOnly)
             exclude = exclude.left(exclude.size() - 1);
 
-        bool removeExcluded = (exclude[0] == ']');
+        bool removeExcluded = (exclude[0] == QLatin1Char(']'));
         if (removeExcluded)
             exclude = exclude.mid(1);
 
-        bool fullPath = exclude.contains('/');
+        bool fullPath = exclude.contains(QLatin1Char('/'));
 
         /* Use QRegularExpression, append to the right pattern */
         auto &bnameFileDir = removeExcluded ? bnameFileDirRemove : bnameFileDirKeep;
@@ -752,7 +749,7 @@ void ExcludedFiles::prepare(const BasePathByteArray & basePath)
             // Make exclude relative to _localPath
             exclude.prepend(relPath);
         }
-        auto regexExclude = convertToRegexpSyntax(QString::fromUtf8(exclude), _wildcardsMatchSlash);
+        auto regexExclude = convertToRegexpSyntax(exclude, _wildcardsMatchSlash);
         if (!fullPath) {
             regexAppend(bnameFileDir, bnameDir, regexExclude, matchDirOnly);
         } else {
@@ -768,7 +765,7 @@ void ExcludedFiles::prepare(const BasePathByteArray & basePath)
     // The empty pattern would match everything - change it to match-nothing
     auto emptyMatchNothing = [](QString &pattern) {
         if (pattern.isEmpty())
-            pattern = "a^";
+            pattern = QStringLiteral("a^");
     };
     emptyMatchNothing(fullFileDirKeep);
     emptyMatchNothing(fullFileDirRemove);
@@ -789,58 +786,58 @@ void ExcludedFiles::prepare(const BasePathByteArray & basePath)
     // If the third group matches, the fullActivatedRegex needs to be applied
     // to the full path.
     _bnameTraversalRegexFile[basePath].setPattern(
-        "^(?P<exclude>" + bnameFileDirKeep + ")$|"
-        + "^(?P<excluderemove>" + bnameFileDirRemove + ")$|"
-        + "^(?P<trigger>" + bnameTriggerFileDir + ")$");
+        QStringLiteral("^(?P<exclude>%1)$|"
+                       "^(?P<excluderemove>%2)$|"
+                       "^(?P<trigger>%3)$")
+            .arg(bnameFileDirKeep, bnameFileDirRemove, bnameTriggerFileDir));
     _bnameTraversalRegexDir[basePath].setPattern(
-        "^(?P<exclude>" + bnameFileDirKeep + "|" + bnameDirKeep + ")$|"
-        + "^(?P<excluderemove>" + bnameFileDirRemove + "|" + bnameDirRemove + ")$|"
-        + "^(?P<trigger>" + bnameTriggerFileDir + "|" + bnameTriggerDir + ")$");
+        QStringLiteral("^(?P<exclude>%1|%2)$|"
+                       "^(?P<excluderemove>%3|%4)$|"
+                       "^(?P<trigger>%5|%6)$")
+            .arg(bnameFileDirKeep, bnameDirKeep, bnameFileDirRemove, bnameDirRemove, bnameTriggerFileDir, bnameTriggerDir));
 
     // The full traveral regex is applied to the full path if the trigger capture of
     // the bname regex matches. Its basic form is (exclude)|(excluderemove)".
     // This pattern can be much simpler than fullRegex since we can assume a traversal
     // situation and doesn't need to look for bname patterns in parent paths.
     _fullTraversalRegexFile[basePath].setPattern(
-        QLatin1String("")
         // Full patterns are anchored to the beginning
-        + "^(?P<exclude>" + fullFileDirKeep + ")(?:$|/)"
-        + "|"
-        + "^(?P<excluderemove>" + fullFileDirRemove + ")(?:$|/)");
+        QStringLiteral("^(?P<exclude>%1)(?:$|/)"
+                       "|"
+                       "^(?P<excluderemove>%2)(?:$|/)")
+            .arg(fullFileDirKeep, fullFileDirRemove));
     _fullTraversalRegexDir[basePath].setPattern(
-        QLatin1String("")
-        + "^(?P<exclude>" + fullFileDirKeep + "|" + fullDirKeep + ")(?:$|/)"
-        + "|"
-        + "^(?P<excluderemove>" + fullFileDirRemove + "|" + fullDirRemove + ")(?:$|/)");
+        QStringLiteral("^(?P<exclude>%1|%2)(?:$|/)"
+                       "|"
+                       "^(?P<excluderemove>%3|%4)(?:$|/)")
+            .arg(fullFileDirKeep, fullDirKeep, fullFileDirRemove, fullDirRemove));
 
     // The full regex is applied to the full path and incorporates both bname and
     // full-path patterns. It has the form "(exclude)|(excluderemove)".
     _fullRegexFile[basePath].setPattern(
-        QLatin1String("(?P<exclude>")
-        // Full patterns are anchored to the beginning
-        + "^(?:" + fullFileDirKeep + ")(?:$|/)" + "|"
-        // Simple bname patterns can be any path component
-        + "(?:^|/)(?:" + bnameFileDirKeep + ")(?:$|/)" + "|"
-        // When checking a file for exclusion we must check all parent paths
-        // against the dir-only patterns as well.
-        + "(?:^|/)(?:" + bnameDirKeep + ")/"
-        + ")"
-        + "|"
-        + "(?P<excluderemove>"
-        + "^(?:" + fullFileDirRemove + ")(?:$|/)" + "|"
-        + "(?:^|/)(?:" + bnameFileDirRemove + ")(?:$|/)" + "|"
-        + "(?:^|/)(?:" + bnameDirRemove + ")/"
-        + ")");
+        QStringLiteral("(?P<exclude>"
+                       // Full patterns are anchored to the beginning
+                       "^(?:%1)(?:$|/)|"
+                       // Simple bname patterns can be any path component
+                       "(?:^|/)(?:%2)(?:$|/)|"
+                       // When checking a file for exclusion we must check all parent paths
+                       // against the dir-only patterns as well.
+                       "(?:^|/)(?:%3)/)"
+                       "|"
+                       "(?P<excluderemove>"
+                       "^(?:%4)(?:$|/)|"
+                       "(?:^|/)(?:%5)(?:$|/)|"
+                       "(?:^|/)(?:%6)/)")
+            .arg(fullFileDirKeep, bnameFileDirKeep, bnameDirKeep, fullFileDirRemove, bnameFileDirRemove, bnameDirRemove));
     _fullRegexDir[basePath].setPattern(
-        QLatin1String("(?P<exclude>")
-        + "^(?:" + fullFileDirKeep + "|" + fullDirKeep + ")(?:$|/)" + "|"
-        + "(?:^|/)(?:" + bnameFileDirKeep + "|" + bnameDirKeep + ")(?:$|/)"
-        + ")"
-        + "|"
-        + "(?P<excluderemove>"
-        + "^(?:" + fullFileDirRemove + "|" + fullDirRemove + ")(?:$|/)" + "|"
-        + "(?:^|/)(?:" + bnameFileDirRemove + "|" + bnameDirRemove + ")(?:$|/)"
-        + ")");
+        QStringLiteral("(?P<exclude>"
+                       "^(?:%1|%2)(?:$|/)|"
+                       "(?:^|/)(?:%3|%4)(?:$|/))"
+                       "|"
+                       "(?P<excluderemove>"
+                       "^(?:%5|%6)(?:$|/)|"
+                       "(?:^|/)(?:%7|%8)(?:$|/))")
+            .arg(fullFileDirKeep, fullDirKeep, bnameFileDirKeep, bnameDirKeep, fullFileDirRemove, fullDirRemove, bnameFileDirRemove, bnameDirRemove));
 
     QRegularExpression::PatternOptions patternOptions = QRegularExpression::NoPatternOption;
     if (OCC::Utility::fsCasePreserving())
