@@ -43,7 +43,8 @@ enum CSYNC_EXCLUDE_TYPE {
   CSYNC_FILE_EXCLUDE_HIDDEN,
   CSYNC_FILE_EXCLUDE_STAT_FAILED,
   CSYNC_FILE_EXCLUDE_CONFLICT,
-  CSYNC_FILE_EXCLUDE_CANNOT_ENCODE
+  CSYNC_FILE_EXCLUDE_CANNOT_ENCODE,
+  CSYNC_FILE_EXCLUDE_SERVER_BLACKLISTED,
 };
 
 class ExcludedFilesTest;
@@ -65,7 +66,9 @@ class OCSYNC_EXPORT ExcludedFiles : public QObject
 {
     Q_OBJECT
 public:
-    ExcludedFiles(QString localPath = "/");
+    using Version = std::tuple<int, int, int>;
+
+    explicit ExcludedFiles(const QString &localPath = QStringLiteral("/"));
     ~ExcludedFiles();
 
     /**
@@ -100,8 +103,8 @@ public:
      * Primarily used in tests. Patterns added this way are preserved when
      * reloadExcludeFiles() is called.
      */
-    void addManualExclude(const QByteArray &expr);
-    void addManualExclude(const QByteArray &expr, const QByteArray &basePath);
+    void addManualExclude(const QString &expr);
+    void addManualExclude(const QString &expr, const QString &basePath);
 
     /**
      * Removes all manually added exclude patterns.
@@ -116,35 +119,9 @@ public:
     void setWildcardsMatchSlash(bool onoff);
 
     /**
-     * Generate a hook for traversal exclude pattern matching
-     * that csync can use.
-     *
-     * Careful: The function will only be valid for as long as this
-     * ExcludedFiles instance stays alive.
+     * Sets the client version, only used for testing.
      */
-    auto csyncTraversalMatchFun()
-        -> std::function<CSYNC_EXCLUDE_TYPE(const char *path, ItemType filetype)>;
-
-public slots:
-    /**
-     * Reloads the exclude patterns from the registered paths.
-     */
-    bool reloadExcludeFiles();
-    /**
-     * Loads the exclude patterns from file the registered base paths.
-     */
-    bool loadExcludeFile(const QByteArray & basePath, const QString & file);
-
-private:
-    /**
-     * @brief Match the exclude pattern against the full path.
-     *
-     * @param Path is folder-relative, should not start with a /.
-     *
-     * Note that this only matches patterns. It does not check whether the file
-     * or directory pointed to is hidden (or whether it even exists).
-     */
-    CSYNC_EXCLUDE_TYPE fullPatternMatch(const char *path, ItemType filetype) const;
+    void setClientVersion(Version version);
 
     /**
      * @brief Check if the given path should be excluded in a traversal situation.
@@ -161,27 +138,60 @@ private:
      * Note that this only matches patterns. It does not check whether the file
      * or directory pointed to is hidden (or whether it even exists).
      */
-    CSYNC_EXCLUDE_TYPE traversalPatternMatch(const char *path, ItemType filetype);
+    CSYNC_EXCLUDE_TYPE traversalPatternMatch(const QString &path, ItemType filetype);
+
+public slots:
+    /**
+     * Reloads the exclude patterns from the registered paths.
+     */
+    bool reloadExcludeFiles();
+    /**
+     * Loads the exclude patterns from file the registered base paths.
+     */
+    bool loadExcludeFile(const QString &basePath, const QString &file);
+
+private:
+    /**
+     * Returns true if the version directive indicates the next line
+     * should be skipped.
+     *
+     * A version directive has the form "#!version <op> <version>"
+     * where <op> can be <, <=, ==, >, >= and <version> can be any version
+     * like 2.5.0.
+     *
+     * Example:
+     *
+     * #!version < 2.5.0
+     * myexclude
+     *
+     * Would enable the "myexclude" pattern only for versions before 2.5.0.
+     */
+    bool versionDirectiveKeepNextLine(const QByteArray &directive) const;
+
+    /**
+     * @brief Match the exclude pattern against the full path.
+     *
+     * @param Path is folder-relative, should not start with a /.
+     *
+     * Note that this only matches patterns. It does not check whether the file
+     * or directory pointed to is hidden (or whether it even exists).
+     */
+    CSYNC_EXCLUDE_TYPE fullPatternMatch(const QString &path, ItemType filetype) const;
 
     // Our BasePath need to end with '/'
-    class BasePathByteArray : public QByteArray
+    class BasePathString : public QString
     {
     public:
-        BasePathByteArray(QByteArray && other)
-            : QByteArray(std::move(other))
+        BasePathString(QString &&other)
+            : QString(std::move(other))
         {
-            Q_ASSERT(this->endsWith('/'));
+            Q_ASSERT(endsWith(QLatin1Char('/')));
         }
 
-        BasePathByteArray(const QByteArray & other)
-            : QByteArray(other)
-        {
-            Q_ASSERT(this->endsWith('/'));
-        }
-
-        BasePathByteArray(const char * data, int size = -1)
-            : BasePathByteArray(QByteArray(data, size))
+        BasePathString(const QString &other)
+            : QString(other)
         {
+            Q_ASSERT(endsWith(QLatin1Char('/')));
         }
     };
 
@@ -214,28 +224,31 @@ private:
      * full matcher would exclude. Example: "b" is excluded. traversal("b/c")
      * returns not-excluded because "c" isn't a bname activation pattern.
      */
-    void prepare(const BasePathByteArray & basePath);
+    void prepare(const BasePathString &basePath);
 
     void prepare();
 
+    static QString extractBnameTrigger(const QString &exclude, bool wildcardsMatchSlash);
+    static QString convertToRegexpSyntax(QString exclude, bool wildcardsMatchSlash);
 
     QString _localPath;
+
     /// Files to load excludes from
-    QMap<BasePathByteArray, QList<QString>> _excludeFiles;
+    QMap<BasePathString, QStringList> _excludeFiles;
 
     /// Exclude patterns added with addManualExclude()
-    QMap<BasePathByteArray, QList<QByteArray>> _manualExcludes;
+    QMap<BasePathString, QStringList> _manualExcludes;
 
     /// List of all active exclude patterns
-    QMap<BasePathByteArray, QList<QByteArray>> _allExcludes;
+    QMap<BasePathString, QStringList> _allExcludes;
 
     /// see prepare()
-    QMap<BasePathByteArray, QRegularExpression> _bnameTraversalRegexFile;
-    QMap<BasePathByteArray, QRegularExpression> _bnameTraversalRegexDir;
-    QMap<BasePathByteArray, QRegularExpression> _fullTraversalRegexFile;
-    QMap<BasePathByteArray, QRegularExpression> _fullTraversalRegexDir;
-    QMap<BasePathByteArray, QRegularExpression> _fullRegexFile;
-    QMap<BasePathByteArray, QRegularExpression> _fullRegexDir;
+    QMap<BasePathString, QRegularExpression> _bnameTraversalRegexFile;
+    QMap<BasePathString, QRegularExpression> _bnameTraversalRegexDir;
+    QMap<BasePathString, QRegularExpression> _fullTraversalRegexFile;
+    QMap<BasePathString, QRegularExpression> _fullTraversalRegexDir;
+    QMap<BasePathString, QRegularExpression> _fullRegexFile;
+    QMap<BasePathString, QRegularExpression> _fullRegexDir;
 
     bool _excludeConflictFiles = true;
 
@@ -247,7 +260,13 @@ private:
      */
     bool _wildcardsMatchSlash = false;
 
-    friend class ExcludedFilesTest;
+    /**
+     * The client version. Used to evaluate version-dependent excludes,
+     * see versionDirectiveKeepNextLine().
+     */
+    Version _clientVersion;
+
+    friend class TestExcludedFiles;
 };
 
 #endif /* _CSYNC_EXCLUDE_H */
