@@ -51,6 +51,7 @@ class FolderStatusModel;
 class AccountSettings : public QWidget
 {
     Q_OBJECT
+    Q_PROPERTY(AccountState* accountState MEMBER _accountState)
 
 public:
     explicit AccountSettings(AccountState *accountState, QWidget *parent = nullptr);
@@ -62,7 +63,7 @@ signals:
     void folderChanged();
     void openFolderAlias(const QString &);
     void showIssuesList(AccountState *account);
-    void requesetMnemonic();
+    void requestMnemonic();
     void removeAccountFolders(AccountState *account);
     void styleChanged();
 
@@ -71,12 +72,12 @@ public slots:
     void slotUpdateQuota(qint64 total, qint64 used);
     void slotAccountStateChanged();
     void slotStyleChanged();
-
     AccountState *accountsState() { return _accountState; }
+    void slotHideSelectiveSyncWidget();
 
 protected slots:
     void slotAddFolder();
-    void slotEnableCurrentFolder();
+    void slotEnableCurrentFolder(bool terminate = false);
     void slotScheduleCurrentFolder();
     void slotScheduleCurrentFolderForceRemoteDiscovery();
     void slotForceSyncCurrentFolder();
@@ -84,13 +85,18 @@ protected slots:
     void slotOpenCurrentFolder(); // sync folder
     void slotOpenCurrentLocalSubFolder(); // selected subfolder in sync folder
     void slotEditCurrentIgnoredFiles();
+    void slotOpenMakeFolderDialog();
     void slotEditCurrentLocalIgnoredFiles();
+    void slotEnableVfsCurrentFolder();
+    void slotDisableVfsCurrentFolder();
+    void slotSetCurrentFolderAvailability(PinState state);
+    void slotSetSubFolderAvailability(Folder *folder, const QString &path, PinState state);
     void slotFolderWizardAccepted();
     void slotFolderWizardRejected();
     void slotDeleteAccount();
     void slotToggleSignInState();
     void refreshSelectiveSyncStatus();
-    void slotMarkSubfolderEncrypted(const FolderStatusModel::SubFolderInfo* folderInfo);
+    void slotMarkSubfolderEncrypted(FolderStatusModel::SubFolderInfo *folderInfo);
     void slotSubfolderContextMenuRequested(const QModelIndex& idx, const QPoint& point);
     void slotCustomContextMenuRequested(const QPoint &);
     void slotFolderListClicked(const QModelIndex &indx);
@@ -102,6 +108,9 @@ protected slots:
     void slotNewMnemonicGenerated();
     void slotEncryptFolderFinished(int status);
 
+    void slotSelectiveSyncChanged(const QModelIndex &topLeft, const QModelIndex &bottomRight,
+                                  const QVector<int> &roles);
+
 private:
     void showConnectionLabel(const QString &message,
         QStringList errors = QStringList());
