@@ -21,6 +21,7 @@
 #include <QPointer>
 #include "connectionvalidator.h"
 #include "creds/abstractcredentials.h"
+#include "userstatus.h"
 #include <memory>
 
 class QSettings;
@@ -42,6 +43,8 @@ using AccountAppList = QList<AccountApp *>;
 class AccountState : public QObject, public QSharedData
 {
     Q_OBJECT
+    Q_PROPERTY(AccountPtr account MEMBER _account)
+
 public:
     enum State {
         /// Not even attempting to connect, most likely because the
@@ -134,7 +137,7 @@ public:
      *  the server to validate the connection if the last successful etag job
      *  was not so long ago.
      */
-    void tagLastSuccessfullETagRequest();
+    void tagLastSuccessfullETagRequest(const QDateTime &tp);
 
     /** Saves the ETag Response header from the last Notifications api
      * request with statusCode 200.
@@ -159,6 +162,32 @@ public:
     ///Asks for user credentials
     void handleInvalidCredentials();
 
+    /** Returns the user status (Online, Dnd, Away, Offline, Invisible)
+     *  https://gist.github.com/georgehrke/55a0412007f13be1551d1f9436a39675
+    */
+    UserStatus::Status status() const;
+
+    /** Returns the user status Message (emoji + text)
+    */
+    QString statusMessage() const;
+
+    /** Returns the user status icon url
+    */
+    QUrl statusIcon() const;
+
+    /** Returns the notifications status retrieved by the notificatons endpoint
+     *  https://github.com/nextcloud/desktop/issues/2318#issuecomment-680698429
+    */
+    bool isDesktopNotificationsAllowed() const;
+
+    /** Set desktop notifications status retrieved by the notificatons endpoint
+    */
+    void setDesktopNotificationsAllowed(bool isAllowed);
+
+    /** Fetch the user status (status, icon, message)
+    */
+    void fetchUserStatus();
+
 public slots:
     /// Triggers a ping to the server to update state and
     /// connection status and errors.
@@ -169,9 +198,10 @@ private:
     void fetchNavigationApps();
 
 signals:
-    void stateChanged(int state);
+    void stateChanged(State state);
     void isConnectedChanged();
     void hasFetchedNavigationApps();
+    void statusChanged();
 
 protected Q_SLOTS:
     void slotConnectionValidatorResult(ConnectionValidator::Status status, const QStringList &errors);
@@ -193,7 +223,7 @@ private:
     ConnectionStatus _connectionStatus;
     QStringList _connectionErrors;
     bool _waitingForNewCredentials;
-    QElapsedTimer _timeSinceLastETagCheck;
+    QDateTime _timeOfLastETagCheck;
     QPointer<ConnectionValidator> _connectionValidator;
     QByteArray _notificationsEtagResponseHeader;
     QByteArray _navigationAppsEtagResponseHeader;
@@ -221,6 +251,8 @@ private:
      */
     AccountAppList _apps;
 
+    UserStatus *_userStatus;
+    bool _isDesktopNotificationsAllowed;
 };
 
 class AccountApp : public QObject
