@@ -13,12 +13,10 @@
  * for more details.
  */
 
-extern "C" {
-    #include <glib.h>
-    #include <gio.h>
-    #include <cloudprovidersaccountexporter.h>
-    #include <cloudprovidersproviderexporter.h>
-}
+#include <glib.h>
+#include <gio.h>
+#include <cloudprovidersaccountexporter.h>
+#include <cloudprovidersproviderexporter.h>
 
 #include "cloudproviderwrapper.h"
 #include <account.h>
@@ -123,8 +121,8 @@ void CloudProviderWrapper::slotUpdateProgress(const QString &folder, const Progr
     if (!progress._currentDiscoveredRemoteFolder.isEmpty()) {
         msg =  tr("Checking for changes in '%1'").arg(progress._currentDiscoveredRemoteFolder);
     } else if (progress.totalSize() == 0) {
-        quint64 currentFile = progress.currentFile();
-        quint64 totalFileCount = qMax(progress.totalFiles(), currentFile);
+        qint64 currentFile = progress.currentFile();
+        qint64 totalFileCount = qMax(progress.totalFiles(), currentFile);
         if (progress.trustEta()) {
             msg = tr("Syncing %1 of %2  (%3 left)")
                     .arg(currentFile)
