@@ -68,7 +68,13 @@ public:
     int setupFolders();
     int setupFoldersMigration();
 
-    OCC::Folder::Map map();
+    /**
+     * Returns a list of keys that can't be read because they are from
+     * future versions.
+     */
+    static void backwardMigrationSettingsKeys(QStringList *deleteKeys, QStringList *ignoreKeys);
+
+    const Folder::Map &map() const;
 
     /** Adds a folder for an account, ensures the journal is gone and saves it in the settings.
       */
@@ -158,9 +164,22 @@ public:
 
     /**
      * Access to the currently syncing folder.
+     *
+     * Note: This is only the folder that's currently syncing *as-scheduled*. There
+     * may be externally-managed syncs such as from placeholder hydrations.
+     *
+     * See also isAnySyncRunning()
      */
     Folder *currentSyncFolder() const;
 
+    /**
+     * Returns true if any folder is currently syncing.
+     *
+     * This might be a FolderMan-scheduled sync, or a externally
+     * managed sync like a placeholder hydration.
+     */
+    bool isAnySyncRunning() const;
+
     /** Removes all folders */
     int unloadAndDeleteAllFolders();
 
@@ -182,13 +201,6 @@ public:
     void setDirtyProxy();
     void setDirtyNetworkLimits();
 
-    /**
-     * Terminates the current folder sync.
-     *
-     * It does not switch the folder to paused state.
-     */
-    void terminateSyncProcess();
-
 signals:
     /**
       * signal to indicate a folder has changed its sync state.
@@ -252,6 +264,8 @@ private slots:
     void slotStartScheduledFolderSync();
     void slotEtagPollTimerTimeout();
 
+    void slotAccountRemoved(AccountState *accountState);
+
     void slotRemoveFoldersForAccount(AccountState *accountState);
 
     // Wraps the Folder::syncStateChange() signal into the
@@ -276,12 +290,16 @@ private slots:
      */
     void slotScheduleFolderByTime();
 
+    void slotSetupPushNotifications(const Folder::Map &);
+    void slotProcessFilesPushNotification(Account *account);
+    void slotConnectToPushNotifications(Account *account);
+
 private:
     /** Adds a new folder, does not add it to the account settings and
      *  does not set an account on the new folder.
       */
     Folder *addFolderInternal(FolderDefinition folderDefinition,
-        AccountState *accountState);
+        AccountState *accountState, std::unique_ptr<Vfs> vfs);
 
     /* unloads a folder object, does not delete it */
     void unloadFolder(Folder *);
@@ -299,7 +317,12 @@ private:
     // restarts the application (Linux only)
     void restartApplication();
 
-    void setupFoldersHelper(QSettings &settings, AccountStatePtr account, bool backwardsCompatible);
+    void setupFoldersHelper(QSettings &settings, AccountStatePtr account, const QStringList &ignoreKeys, bool backwardsCompatible, bool foldersWithPlaceholders);
+
+    void runEtagJobsIfPossible(const QList<Folder *> &folderMap);
+    void runEtagJobIfPossible(Folder *folder);
+
+    bool pushNotificationsFilesReady(Account *account);
 
     QSet<Folder *> _disabledFolders;
     Folder::Map _folderMap;
@@ -308,6 +331,9 @@ private:
     QPointer<Folder> _lastSyncFolder;
     bool _syncEnabled = true;
 
+    /// Folder aliases from the settings that weren't read
+    QSet<QString> _additionalBlockedFolderAliases;
+
     /// Starts regular etag query jobs
     QTimer _etagPollTimer;
     /// The currently running etag query
