@@ -12,15 +12,16 @@
  * for more details.
  */
 
-
 #ifndef SOCKETAPI_H
 #define SOCKETAPI_H
 
 #include "syncfileitem.h"
-#include "syncfilestatus.h"
+#include "common/syncfilestatus.h"
 #include "sharedialog.h" // for the ShareDialogStartPage
 #include "common/syncjournalfilerecord.h"
 
+#include "config.h"
+
 #if defined(Q_OS_MAC)
 #include "socketapisocket_mac.h"
 #else
@@ -38,6 +39,9 @@ class SyncFileStatus;
 class Folder;
 class SocketListener;
 class DirectEditor;
+class SocketApiJob;
+
+Q_DECLARE_LOGGING_CATEGORY(lcSocketApi)
 
 /**
  * @brief The SocketApi class
@@ -81,10 +85,16 @@ private:
         SyncJournalFileRecord journalRecord() const;
         FileData parentFolder() const;
 
+        // Relative path of the file locally, without any vfs suffix
+        QString folderRelativePathNoVfsSuffix() const;
+
         Folder *folder;
+        // Absolute path of the file locally. (May be a virtual file)
         QString localPath;
+        // Relative path of the file locally, as in the DB. (May be a virtual file)
         QString folderRelativePath;
-        QString accountRelativePath;
+        // Path of the file on the server (In case of virtual file, it points to the actual file)
+        QString serverRelativePath;
     };
 
     void broadcastMessage(const QString &msg, bool doWait = false);
@@ -106,6 +116,8 @@ private:
     Q_INVOKABLE void command_COPY_PRIVATE_LINK(const QString &localFile, SocketListener *listener);
     Q_INVOKABLE void command_EMAIL_PRIVATE_LINK(const QString &localFile, SocketListener *listener);
     Q_INVOKABLE void command_OPEN_PRIVATE_LINK(const QString &localFile, SocketListener *listener);
+    Q_INVOKABLE void command_MAKE_AVAILABLE_LOCALLY(const QString &filesArg, SocketListener *listener);
+    Q_INVOKABLE void command_MAKE_ONLINE_ONLY(const QString &filesArg, SocketListener *listener);
     Q_INVOKABLE void command_RESOLVE_CONFLICT(const QString &localFile, SocketListener *listener);
     Q_INVOKABLE void command_DELETE_ITEM(const QString &localFile, SocketListener *listener);
     Q_INVOKABLE void command_MOVE_ITEM(const QString &localFile, SocketListener *listener);
@@ -139,6 +151,16 @@ private:
     Q_INVOKABLE void command_EDIT(const QString &localFile, SocketListener *listener);
     DirectEditor* getDirectEditorForLocalFile(const QString &localFile);
 
+#if GUI_TESTING
+    Q_INVOKABLE void command_ASYNC_ASSERT_ICON_IS_EQUAL(const QSharedPointer<SocketApiJob> &job);
+    Q_INVOKABLE void command_ASYNC_LIST_WIDGETS(const QSharedPointer<SocketApiJob> &job);
+    Q_INVOKABLE void command_ASYNC_INVOKE_WIDGET_METHOD(const QSharedPointer<SocketApiJob> &job);
+    Q_INVOKABLE void command_ASYNC_GET_WIDGET_PROPERTY(const QSharedPointer<SocketApiJob> &job);
+    Q_INVOKABLE void command_ASYNC_SET_WIDGET_PROPERTY(const QSharedPointer<SocketApiJob> &job);
+    Q_INVOKABLE void command_ASYNC_WAIT_FOR_WIDGET_SIGNAL(const QSharedPointer<SocketApiJob> &job);
+    Q_INVOKABLE void command_ASYNC_TRIGGER_MENU_ACTION(const QSharedPointer<SocketApiJob> &job);
+#endif
+
     QString buildRegisterPathMessage(const QString &path);
 
     QSet<QString> _registeredAliases;
@@ -146,4 +168,5 @@ private:
     SocketApiServer _localServer;
 };
 }
+
 #endif // SOCKETAPI_H
