@@ -19,6 +19,8 @@ class User : public QObject
     Q_OBJECT
     Q_PROPERTY(QString name READ name NOTIFY nameChanged)
     Q_PROPERTY(QString server READ server CONSTANT)
+    Q_PROPERTY(QUrl statusIcon READ statusIcon NOTIFY statusChanged)
+    Q_PROPERTY(QString statusMessage READ statusMessage NOTIFY statusChanged)
     Q_PROPERTY(bool hasLocalFolder READ hasLocalFolder NOTIFY hasLocalFolderChanged)
     Q_PROPERTY(bool serverHasTalk READ serverHasTalk NOTIFY serverHasTalkChanged)
     Q_PROPERTY(QString avatar READ avatarUrl NOTIFY avatarChanged)
@@ -45,6 +47,10 @@ public:
     void logout() const;
     void removeAccount() const;
     QString avatarUrl() const;
+    bool isDesktopNotificationsAllowed() const;
+    UserStatus::Status status() const;
+    QString statusMessage() const;
+    QUrl statusIcon() const;
 
 signals:
     void guiLog(const QString &, const QString &);
@@ -53,6 +59,7 @@ signals:
     void serverHasTalkChanged();
     void avatarChanged();
     void accountStateChanged(int state);
+    void statusChanged();
 
 public slots:
     void slotItemCompleted(const QString &folder, const SyncFileItemPtr &item);
@@ -67,10 +74,23 @@ public slots:
     void slotRefreshNotifications();
     void slotRefreshActivities();
     void slotRefresh();
+    void slotRefreshUserStatus();
     void slotRefreshImmediately();
     void setNotificationRefreshInterval(std::chrono::milliseconds interval);
     void slotRebuildNavigationAppList();
 
+private:
+    void slotPushNotificationsReady();
+    void slotDisconnectPushNotifications();
+    void slotReceivedPushNotification(Account *account);
+    void slotReceivedPushActivity(Account *account);
+
+    void connectPushNotifications() const;
+    bool checkPushNotificationsAreReady() const;
+
+    bool isActivityOfCurrentAccount(const Folder *folder) const;
+    bool isUnsolvableConflict(const SyncFileItemPtr &item) const;
+
 private:
     AccountStatePtr _account;
     bool _isCurrentUser;
@@ -120,6 +140,7 @@ public:
     Q_INVOKABLE bool currentUserHasLocalFolder();
     int currentUserId() const;
     Q_INVOKABLE bool isUserConnected(const int &id);
+    Q_INVOKABLE QUrl statusIcon(int id);
     Q_INVOKABLE void switchCurrentUser(const int &id);
     Q_INVOKABLE void login(const int &id);
     Q_INVOKABLE void logout(const int &id);
@@ -130,6 +151,8 @@ public:
     enum UserRoles {
         NameRole = Qt::UserRole + 1,
         ServerRole,
+        StatusIconRole,
+        StatusMessageRole,
         AvatarRole,
         IsCurrentUserRole,
         IsConnectedRole,
