@@ -25,6 +25,8 @@ class QProgressIndicator;
 
 namespace OCC {
 
+class OwncloudWizard;
+
 /**
  * @brief The OwncloudAdvancedSetupPage class
  * @ingroup gui
@@ -33,7 +35,7 @@ class OwncloudAdvancedSetupPage : public QWizardPage
 {
     Q_OBJECT
 public:
-    OwncloudAdvancedSetupPage();
+    OwncloudAdvancedSetupPage(OwncloudWizard *wizard);
 
     bool isComplete() const override;
     void initializePage() override;
@@ -41,6 +43,7 @@ public:
     bool validatePage() override;
     QString localFolder() const;
     QStringList selectiveSyncBlacklist() const;
+    bool useVirtualFileSync() const;
     bool isConfirmBigFolderChecked() const;
     void setRemoteFolder(const QString &remoteFolder);
     void setMultipleFoldersExist(bool exist);
@@ -57,9 +60,12 @@ private slots:
     void slotSelectFolder();
     void slotSyncEverythingClicked();
     void slotSelectiveSyncClicked();
+    void slotVirtualFileSyncClicked();
     void slotQuotaRetrieved(const QVariantMap &result);
 
 private:
+    void setRadioChecked(QRadioButton *radio);
+
     void setupCustomization();
     void updateStatus();
     bool dataChanged();
@@ -69,6 +75,17 @@ private:
     qint64 availableLocalSpace() const;
     QString checkLocalSpace(qint64 remoteSize) const;
     void customizeStyle();
+    void setServerAddressLabelUrl(const QUrl &url);
+    void setLocalFolderPushButtonPath(const QString &path);
+    void styleSyncLogo();
+    void styleLocalFolderLabel();
+    void setResolutionGuiVisible(bool value);
+    void setupResoultionWidget();
+    void fetchUserAvatar();
+    void fetchUserData();
+
+    // TODO: remove when UX decision is made
+    void refreshVirtualFilesAvailibility(const QString &path);
 
     Ui_OwncloudAdvancedSetupPage _ui;
     bool _checking = false;
@@ -79,6 +96,7 @@ private:
     QStringList _selectiveSyncBlacklist;
     qint64 _rSize = -1;
     qint64 _rSelectedSize = -1;
+    OwncloudWizard *_ocWizard;
 };
 
 } // namespace OCC
