@@ -78,8 +78,9 @@ signals:
     void determineAuthType(const QString &);
 
 private:
-    bool urlHasChanged();
+    void setLogo();
     void customizeStyle();
+    void setupServerAddressDescriptionLabel();
 
     Ui_OwncloudSetupPage _ui;
 
