@@ -14,26 +14,29 @@
  */
 
 #include "account.h"
+#include "config.h"
 #include "configfile.h"
 #include "theme.h"
 #include "owncloudgui.h"
 
 #include "wizard/owncloudwizard.h"
+#include "wizard/welcomepage.h"
 #include "wizard/owncloudsetuppage.h"
 #include "wizard/owncloudhttpcredspage.h"
 #include "wizard/owncloudoauthcredspage.h"
-#ifndef NO_SHIBBOLETH
-#include "wizard/owncloudshibbolethcredspage.h"
-#endif
 #include "wizard/owncloudadvancedsetuppage.h"
 #include "wizard/owncloudwizardresultpage.h"
 #include "wizard/webviewpage.h"
 #include "wizard/flow2authcredspage.h"
 
+#include "common/vfs.h"
+
 #include "QProgressIndicator.h"
 
 #include <QtCore>
 #include <QtGui>
+#include <QMessageBox>
+#include <owncloudgui.h>
 
 #include <cstdlib>
 
@@ -44,25 +47,23 @@ Q_LOGGING_CATEGORY(lcWizard, "nextcloud.gui.wizard", QtInfoMsg)
 OwncloudWizard::OwncloudWizard(QWidget *parent)
     : QWizard(parent)
     , _account(nullptr)
+    , _welcomePage(new WelcomePage(this))
     , _setupPage(new OwncloudSetupPage(this))
     , _httpCredsPage(new OwncloudHttpCredsPage(this))
     , _browserCredsPage(new OwncloudOAuthCredsPage)
-#ifndef NO_SHIBBOLETH
-    , _shibbolethCredsPage(new OwncloudShibbolethCredsPage)
-#endif
     , _flow2CredsPage(new Flow2AuthCredsPage)
-    , _advancedSetupPage(new OwncloudAdvancedSetupPage)
+    , _advancedSetupPage(new OwncloudAdvancedSetupPage(this))
     , _resultPage(new OwncloudWizardResultPage)
     , _webViewPage(new WebViewPage(this))
 {
+    setObjectName("owncloudWizard");
+
     setWindowFlags(windowFlags() & ~Qt::WindowContextHelpButtonHint);
+    setPage(WizardCommon::Page_Welcome, _welcomePage);
     setPage(WizardCommon::Page_ServerSetup, _setupPage);
     setPage(WizardCommon::Page_HttpCreds, _httpCredsPage);
     setPage(WizardCommon::Page_OAuthCreds, _browserCredsPage);
     setPage(WizardCommon::Page_Flow2AuthCreds, _flow2CredsPage);
-#ifndef NO_SHIBBOLETH
-    setPage(WizardCommon::Page_ShibbolethCreds, _shibbolethCredsPage);
-#endif
     setPage(WizardCommon::Page_AdvancedSetup, _advancedSetupPage);
     setPage(WizardCommon::Page_Result, _resultPage);
     setPage(WizardCommon::Page_WebView, _webViewPage);
@@ -71,16 +72,12 @@ OwncloudWizard::OwncloudWizard(QWidget *parent)
 
     // note: start Id is set by the calling class depending on if the
     // welcome text is to be shown or not.
-    setWizardStyle(QWizard::ModernStyle);
 
     connect(this, &QWizard::currentIdChanged, this, &OwncloudWizard::slotCurrentPageChanged);
     connect(_setupPage, &OwncloudSetupPage::determineAuthType, this, &OwncloudWizard::determineAuthType);
     connect(_httpCredsPage, &OwncloudHttpCredsPage::connectToOCUrl, this, &OwncloudWizard::connectToOCUrl);
     connect(_browserCredsPage, &OwncloudOAuthCredsPage::connectToOCUrl, this, &OwncloudWizard::connectToOCUrl);
     connect(_flow2CredsPage, &Flow2AuthCredsPage::connectToOCUrl, this, &OwncloudWizard::connectToOCUrl);
-#ifndef NO_SHIBBOLETH
-    connect(_shibbolethCredsPage, &OwncloudShibbolethCredsPage::connectToOCUrl, this, &OwncloudWizard::connectToOCUrl);
-#endif
     connect(_webViewPage, &WebViewPage::connectToOCUrl, this, &OwncloudWizard::connectToOCUrl);
     connect(_advancedSetupPage, &OwncloudAdvancedSetupPage::createLocalAndRemoteFolders,
         this, &OwncloudWizard::createLocalAndRemoteFolders);
@@ -88,17 +85,19 @@ OwncloudWizard::OwncloudWizard(QWidget *parent)
 
 
     Theme *theme = Theme::instance();
-    setWindowTitle(tr("%1 Connection Wizard").arg(theme->appNameGUI()));
+    setWindowTitle(tr("Add %1 account").arg(theme->appNameGUI()));
     setWizardStyle(QWizard::ModernStyle);
-    setPixmap(QWizard::BannerPixmap, theme->wizardHeaderBanner());
-    setPixmap(QWizard::LogoPixmap, theme->wizardHeaderLogo());
     setOption(QWizard::NoBackButtonOnStartPage);
     setOption(QWizard::NoBackButtonOnLastPage);
     setOption(QWizard::NoCancelButton);
-    setTitleFormat(Qt::RichText);
-    setSubTitleFormat(Qt::RichText);
     setButtonText(QWizard::CustomButton1, tr("Skip folders configuration"));
 
+    // Change the next buttons size policy since we hide it on the
+    // welcome page but want it to fill it's space that we don't get
+    // flickering when the page changes
+    auto nextButtonSizePolicy = button(QWizard::NextButton)->sizePolicy();
+    nextButtonSizePolicy.setRetainSizeWhenHidden(true);
+    button(QWizard::NextButton)->setSizePolicy(nextButtonSizePolicy);
 
     // Connect styleChanged events to our widgets, so they can adapt (Dark-/Light-Mode switching)
     connect(this, &OwncloudWizard::styleChanged, _setupPage, &OwncloudSetupPage::slotStyleChanged);
@@ -109,6 +108,51 @@ OwncloudWizard::OwncloudWizard(QWidget *parent)
 
     // allow Flow2 page to poll on window activation
     connect(this, &OwncloudWizard::onActivate, _flow2CredsPage, &Flow2AuthCredsPage::slotPollNow);
+
+    adjustWizardSize();
+    centerWindow();
+}
+
+void OwncloudWizard::centerWindow()
+{
+    const auto wizardWindow = window();
+    const auto screen = QGuiApplication::screenAt(wizardWindow->pos())
+        ? QGuiApplication::screenAt(wizardWindow->pos())
+        : QGuiApplication::primaryScreen();
+    const auto screenGeometry = screen->geometry();
+    const auto windowGeometry = wizardWindow->geometry();
+    const auto newWindowPosition = screenGeometry.center() - QPoint(windowGeometry.width() / 2, windowGeometry.height() / 2);
+    wizardWindow->move(newWindowPosition);
+}
+
+
+void OwncloudWizard::adjustWizardSize()
+{
+    const auto pageSizes = calculateWizardPageSizes();
+    const auto longestSide = calculateLongestSideOfWizardPages(pageSizes);
+
+    resize(QSize(longestSide, longestSide));
+}
+
+QList<QSize> OwncloudWizard::calculateWizardPageSizes() const
+{
+    QList<QSize> pageSizes;
+    const auto pIds = pageIds();
+
+    std::transform(pIds.cbegin(), pIds.cend(), std::back_inserter(pageSizes), [this](int pageId) {
+        auto p = page(pageId);
+        p->adjustSize();
+        return p->sizeHint();
+    });
+
+    return pageSizes;
+}
+
+int OwncloudWizard::calculateLongestSideOfWizardPages(const QList<QSize> &pageSizes) const
+{
+    return std::accumulate(std::cbegin(pageSizes), std::cend(pageSizes), 0, [](int current, const QSize &size) {
+        return std::max({ current, size.width(), size.height() });
+    });
 }
 
 void OwncloudWizard::setAccount(AccountPtr account)
@@ -131,6 +175,11 @@ QStringList OwncloudWizard::selectiveSyncBlacklist() const
     return _advancedSetupPage->selectiveSyncBlacklist();
 }
 
+bool OwncloudWizard::useVirtualFileSync() const
+{
+    return _advancedSetupPage->useVirtualFileSync();
+}
+
 bool OwncloudWizard::isConfirmBigFolderChecked() const
 {
     return _advancedSetupPage->isConfirmBigFolderChecked();
@@ -181,12 +230,6 @@ void OwncloudWizard::successfulStep()
         _flow2CredsPage->setConnected();
         break;
 
-#ifndef NO_SHIBBOLETH
-    case WizardCommon::Page_ShibbolethCreds:
-        _shibbolethCredsPage->setConnected();
-        break;
-#endif
-
     case WizardCommon::Page_WebView:
         _webViewPage->setConnected();
         break;
@@ -201,18 +244,15 @@ void OwncloudWizard::successfulStep()
         break;
     }
 
+    ownCloudGui::raiseDialog(this);
     next();
 }
 
 void OwncloudWizard::setAuthType(DetermineAuthTypeJob::AuthType type)
 {
     _setupPage->setAuthType(type);
-#ifndef NO_SHIBBOLETH
-    if (type == DetermineAuthTypeJob::Shibboleth) {
-        _credentialsPage = _shibbolethCredsPage;
-    } else
-#endif
-        if (type == DetermineAuthTypeJob::OAuth) {
+
+    if (type == DetermineAuthTypeJob::OAuth) {
         _credentialsPage = _browserCredsPage;
     } else if (type == DetermineAuthTypeJob::LoginFlowV2) {
         _credentialsPage = _flow2CredsPage;
@@ -229,6 +269,28 @@ void OwncloudWizard::slotCurrentPageChanged(int id)
 {
     qCDebug(lcWizard) << "Current Wizard page changed to " << id;
 
+    const auto setNextButtonAsDefault = [this]() {
+        auto nextButton = qobject_cast<QPushButton *>(button(QWizard::NextButton));
+        if (nextButton) {
+            nextButton->setDefault(true);
+        }
+    };
+
+    if (id == WizardCommon::Page_Welcome) {
+        // Set next button to just hidden so it retains it's layout
+        button(QWizard::NextButton)->setHidden(true);
+        // Need to set it from here, otherwise it has no effect
+        _welcomePage->setLoginButtonDefault();
+    } else if (id == WizardCommon::Page_WebView || id == WizardCommon::Page_Flow2AuthCreds) {
+        setButtonLayout({ QWizard::Stretch, QWizard::BackButton });
+    } else if (id == WizardCommon::Page_AdvancedSetup) {
+        setButtonLayout({ QWizard::Stretch, QWizard::CustomButton1, QWizard::BackButton, QWizard::NextButton });
+        setNextButtonAsDefault();
+    } else {
+        setButtonLayout({ QWizard::Stretch, QWizard::BackButton, QWizard::NextButton });
+        setNextButtonAsDefault();
+    }
+
     if (id == WizardCommon::Page_ServerSetup) {
         emit clearPendingRequests();
     }
@@ -241,7 +303,6 @@ void OwncloudWizard::slotCurrentPageChanged(int id)
         done(Accepted);
     }
 
-    setOption(QWizard::HaveCustomButton1, id == WizardCommon::Page_AdvancedSetup);
     if (id == WizardCommon::Page_AdvancedSetup && (_credentialsPage == _browserCredsPage || _credentialsPage == _flow2CredsPage)) {
         // For OAuth, disable the back button in the Page_AdvancedSetup because we don't want
         // to re-open the browser.
@@ -298,7 +359,7 @@ void OwncloudWizard::changeEvent(QEvent *e)
         emit styleChanged();
         break;
     case QEvent::ActivationChange:
-        if(isActiveWindow())
+        if (isActiveWindow())
             emit onActivate();
         break;
     default:
@@ -311,6 +372,15 @@ void OwncloudWizard::changeEvent(QEvent *e)
 void OwncloudWizard::customizeStyle()
 {
     // HINT: Customize wizard's own style here, if necessary in the future (Dark-/Light-Mode switching)
+
+    // Set background colors
+    auto wizardPalette = palette();
+    const auto backgroundColor = wizardPalette.color(QPalette::Window);
+    wizardPalette.setColor(QPalette::Base, backgroundColor);
+    // Set separator color
+    wizardPalette.setColor(QPalette::Mid, backgroundColor);
+
+    setPalette(wizardPalette);
 }
 
 void OwncloudWizard::bringToTop()
@@ -319,4 +389,45 @@ void OwncloudWizard::bringToTop()
     ownCloudGui::raiseDialog(this);
 }
 
+void OwncloudWizard::askExperimentalVirtualFilesFeature(QWidget *receiver, const std::function<void(bool enable)> &callback)
+{
+    const auto bestVfsMode = bestAvailableVfsMode();
+    QMessageBox *msgBox = nullptr;
+    QPushButton *acceptButton = nullptr;
+    switch (bestVfsMode) {
+    case Vfs::WindowsCfApi:
+        callback(true);
+        return;
+    case Vfs::WithSuffix:
+        msgBox = new QMessageBox(
+            QMessageBox::Warning,
+            tr("Enable experimental feature?"),
+            tr("When the \"virtual files\" mode is enabled no files will be downloaded initially. "
+               "Instead, a tiny \"%1\" file will be created for each file that exists on the server. "
+               "The contents can be downloaded by running these files or by using their context menu."
+               "\n\n"
+               "The virtual files mode is mutually exclusive with selective sync. "
+               "Currently unselected folders will be translated to online-only folders "
+               "and your selective sync settings will be reset."
+               "\n\n"
+               "Switching to this mode will abort any currently running synchronization."
+               "\n\n"
+               "This is a new, experimental mode. If you decide to use it, please report any "
+               "issues that come up.")
+                .arg(APPLICATION_DOTVIRTUALFILE_SUFFIX),
+            QMessageBox::NoButton, receiver);
+        acceptButton = msgBox->addButton(tr("Enable experimental placeholder mode"), QMessageBox::AcceptRole);
+        msgBox->addButton(tr("Stay safe"), QMessageBox::RejectRole);
+        break;
+    case Vfs::Off:
+        Q_UNREACHABLE();
+    }
+
+    connect(msgBox, &QMessageBox::accepted, receiver, [callback, msgBox, acceptButton] {
+        callback(msgBox->clickedButton() == acceptButton);
+        msgBox->deleteLater();
+    });
+    msgBox->open();
+}
+
 } // end namespace
