@@ -17,6 +17,7 @@ public:
     ~WebViewPage();
 
     void initializePage() override;
+    void cleanupPage() override;
     int nextId() const override;
     bool isComplete() const override;
 
@@ -30,6 +31,9 @@ private slots:
     void urlCatched(QString user, QString pass, QString host);
 
 private:
+    void resizeWizard();
+    bool tryToSetWizardSize(int width, int height);
+
     OwncloudWizard *_ocWizard;
     WebView *_webView;
 
@@ -37,6 +41,8 @@ private:
     QString _pass;
 
     bool _useSystemProxy;
+
+    QSize _originalWizardSize;
 };
 
 }
