@@ -26,6 +26,15 @@ namespace OCC {
 
 class DirectEditor;
 
+enum PushNotificationType {
+    None = 0,
+    Files = 1,
+    Activities = 2,
+    Notifications = 4
+};
+Q_DECLARE_FLAGS(PushNotificationTypes, PushNotificationType)
+Q_DECLARE_OPERATORS_FOR_FLAGS(PushNotificationTypes)
+
 /**
  * @brief The Capabilities class represents the capabilities of an ownCloud
  * server
@@ -47,6 +56,13 @@ public:
     bool sharePublicLinkMultiple() const;
     bool shareResharing() const;
     bool chunkingNg() const;
+    bool userStatus() const;
+
+    /// Returns which kind of push notfications are available
+    PushNotificationTypes availablePushNotifications() const;
+
+    /// Websocket url for files push notifications if available
+    QUrl pushNotificationsWebSocketUrl() const;
 
     /// disable parallel upload in chunking
     bool chunkingParallelUploadDisabled() const;
@@ -126,6 +142,11 @@ public:
      */
     QString invalidFilenameRegex() const;
 
+    /**
+     * return the list of filename that should not be uploaded
+     */
+    QStringList blacklistedFiles() const;
+
     /**
      * Whether conflict files should remain local (default) or should be uploaded.
      */
