@@ -21,13 +21,9 @@
 #include <QDirIterator>
 #include <QCoreApplication>
 
-// We use some internals of csync:
-extern "C" int c_utimes(const char *, const struct timeval *);
-
 #include "csync.h"
 #include "vio/csync_vio_local.h"
-#include "std/c_string.h"
-#include "std/c_utf8.h"
+#include "std/c_time.h"
 
 namespace OCC {
 
@@ -46,29 +42,24 @@ bool FileSystem::fileEquals(const QString &fn1, const QString &fn2)
     }
 
     const int BufferSize = 16 * 1024;
-    char buffer1[BufferSize];
-    char buffer2[BufferSize];
-    do {
-        int r = f1.read(buffer1, BufferSize);
-        if (f2.read(buffer2, BufferSize) != r) {
-            // this should normally not happen: the files are supposed to have the same size.
+    QByteArray buffer1(BufferSize, 0);
+    QByteArray buffer2(BufferSize, 0);
+    // the files have the same size, compare all of it
+    while(!f1.atEnd()){
+        f1.read(buffer1.data(), BufferSize);
+        f2.read(buffer2.data(), BufferSize);
+        if (buffer1 != buffer2) {
             return false;
         }
-        if (r <= 0) {
-            return true;
-        }
-        if (memcmp(buffer1, buffer2, r) != 0) {
-            return false;
-        }
-    } while (true);
-    return false;
+    };
+    return true;
 }
 
 time_t FileSystem::getModTime(const QString &filename)
 {
     csync_file_stat_t stat;
     qint64 result = -1;
-    if (csync_vio_local_stat(filename.toUtf8().data(), &stat) != -1
+    if (csync_vio_local_stat(filename, &stat) != -1
         && (stat.modtime != 0)) {
         result = stat.modtime;
     } else {
@@ -84,7 +75,7 @@ bool FileSystem::setModTime(const QString &filename, time_t modTime)
     struct timeval times[2];
     times[0].tv_sec = times[1].tv_sec = modTime;
     times[0].tv_usec = times[1].tv_usec = 0;
-    int rc = c_utimes(filename.toUtf8().data(), times);
+    int rc = c_utimes(filename, times);
     if (rc != 0) {
         qCWarning(lcFileSystem) << "Error setting mtime for" << filename
                                 << "failed: rc" << rc << ", errno:" << errno;
@@ -121,7 +112,7 @@ static qint64 getSizeWithCsync(const QString &filename)
 {
     qint64 result = 0;
     csync_file_stat_t stat;
-    if (csync_vio_local_stat(filename.toUtf8().data(), &stat) != -1) {
+    if (csync_vio_local_stat(filename, &stat) != -1) {
         result = stat.size;
     } else {
         qCWarning(lcFileSystem) << "Could not get size for" << filename << "with csync";
@@ -134,7 +125,7 @@ qint64 FileSystem::getSize(const QString &filename)
 {
 #ifdef Q_OS_WIN
     if (isLnkFile(filename)) {
-        // Use csync to get the file size. Qt seems unable to get at it.
+        // Qt handles .lnk as symlink... https://doc.qt.io/qt-5/qfileinfo.html#details
         return getSizeWithCsync(filename);
     }
 #endif
@@ -189,5 +180,15 @@ bool FileSystem::removeRecursively(const QString &path, const std::function<void
     return allRemoved;
 }
 
+bool FileSystem::getInode(const QString &filename, quint64 *inode)
+{
+    csync_file_stat_t fs;
+    if (csync_vio_local_stat(filename, &fs) == 0) {
+        *inode = fs.inode;
+        return true;
+    }
+    return false;
+}
+
 
 } // namespace OCC
