@@ -17,7 +17,7 @@
 #include "account.h"
 #include "common/syncjournalfilerecord.h"
 #include "propagateuploadencrypted.h"
-#include "propagateremotedelete.h"
+#include "deletejob.h"
 #include "common/asserts.h"
 #include "encryptfolderjob.h"
 
@@ -32,16 +32,7 @@ PropagateRemoteMkdir::PropagateRemoteMkdir(OwncloudPropagator *propagator, const
     : PropagateItemJob(propagator, item)
     , _deleteExisting(false)
     , _uploadEncryptedHelper(nullptr)
-    , _parallelism(FullParallelism)
 {
-    const auto rootPath = [=]() {
-        const auto result = propagator->_remoteFolder;
-        if (result.startsWith('/')) {
-            return result.mid(1);
-        } else {
-            return result;
-        }
-    }();
     const auto path = _item->_file;
     const auto slashPosition = path.lastIndexOf('/');
     const auto parentPath = slashPosition >= 0 ? path.left(slashPosition) : QString();
@@ -49,28 +40,13 @@ PropagateRemoteMkdir::PropagateRemoteMkdir(OwncloudPropagator *propagator, const
     SyncJournalFileRecord parentRec;
     bool ok = propagator->_journal->getFileRecord(parentPath, &parentRec);
     if (!ok) {
-        done(SyncFileItem::NormalError);
         return;
     }
-
-    const auto remoteParentPath = parentRec._e2eMangledName.isEmpty() ? parentPath : parentRec._e2eMangledName;
-    const auto absoluteRemoteParentPath = remoteParentPath.isEmpty() ? rootPath : rootPath + remoteParentPath + '/';
-    const auto account = propagator->account();
-
-    if (account->capabilities().clientSideEncryptionAvailable() &&
-        account->e2e()->isFolderEncrypted(absoluteRemoteParentPath)) {
-        _parallelism = WaitForFinished;
-    }
-}
-
-PropagatorJob::JobParallelism PropagateRemoteMkdir::parallelism()
-{
-    return _parallelism;
 }
 
 void PropagateRemoteMkdir::start()
 {
-    if (propagator()->_abortRequested.fetchAndAddRelaxed(0))
+    if (propagator()->_abortRequested)
         return;
 
     qCDebug(lcPropagateRemoteMkdir) << _item->_file;
@@ -83,7 +59,7 @@ void PropagateRemoteMkdir::start()
     }
 
     _job = new DeleteJob(propagator()->account(),
-        propagator()->_remoteFolder + _item->_file,
+        propagator()->fullRemotePath(_item->_file),
         this);
     connect(static_cast<DeleteJob*>(_job.data()), &DeleteJob::finishedSignal,
             this, &PropagateRemoteMkdir::slotMkdir);
@@ -92,13 +68,13 @@ void PropagateRemoteMkdir::start()
 
 void PropagateRemoteMkdir::slotStartMkcolJob()
 {
-    if (propagator()->_abortRequested.fetchAndAddRelaxed(0))
+    if (propagator()->_abortRequested)
         return;
 
     qCDebug(lcPropagateRemoteMkdir) << _item->_file;
 
     _job = new MkColJob(propagator()->account(),
-        propagator()->_remoteFolder + _item->_file,
+        propagator()->fullRemotePath(_item->_file),
         this);
     connect(_job, SIGNAL(finished(QNetworkReply::NetworkError)), this, SLOT(slotMkcolJobFinished()));
     _job->start();
@@ -109,18 +85,16 @@ void PropagateRemoteMkdir::slotStartEncryptedMkcolJob(const QString &path, const
     Q_UNUSED(path)
     Q_UNUSED(size)
 
-    if (propagator()->_abortRequested.fetchAndAddRelaxed(0))
+    if (propagator()->_abortRequested)
         return;
 
     qDebug() << filename;
     qCDebug(lcPropagateRemoteMkdir) << filename;
 
     auto job = new MkColJob(propagator()->account(),
-                            propagator()->_remoteFolder + filename,
-                            {{"e2e-token", _uploadEncryptedHelper->_folderToken }},
+                            propagator()->fullRemotePath(filename),
+                            {{"e2e-token", _uploadEncryptedHelper->folderToken() }},
                             this);
-    connect(job, qOverload<QNetworkReply::NetworkError>(&MkColJob::finished),
-            _uploadEncryptedHelper, &PropagateUploadEncrypted::unlockFolder);
     connect(job, qOverload<QNetworkReply::NetworkError>(&MkColJob::finished),
             this, &PropagateRemoteMkdir::slotMkcolJobFinished);
     _job = job;
@@ -142,16 +116,61 @@ void PropagateRemoteMkdir::setDeleteExisting(bool enabled)
     _deleteExisting = enabled;
 }
 
+void PropagateRemoteMkdir::finalizeMkColJob(QNetworkReply::NetworkError err, const QString &jobHttpReasonPhraseString, const QString &jobPath)
+{
+    if (_item->_httpErrorCode == 405) {
+        // This happens when the directory already exists. Nothing to do.
+        qDebug(lcPropagateRemoteMkdir) << "Folder" << jobPath << "already exists.";
+    } else if (err != QNetworkReply::NoError) {
+        SyncFileItem::Status status = classifyError(err, _item->_httpErrorCode,
+            &propagator()->_anotherSyncNeeded);
+        done(status, _item->_errorString);
+        return;
+    } else if (_item->_httpErrorCode != 201) {
+        // Normally we expect "201 Created"
+        // If it is not the case, it might be because of a proxy or gateway intercepting the request, so we must
+        // throw an error.
+        done(SyncFileItem::NormalError,
+            tr("Wrong HTTP code returned by server. Expected 201, but received \"%1 %2\".")
+                .arg(_item->_httpErrorCode)
+                .arg(jobHttpReasonPhraseString));
+        return;
+    }
+
+    if (_item->_fileId.isEmpty()) {
+        // Owncloud 7.0.0 and before did not have a header with the file id.
+        // (https://github.com/owncloud/core/issues/9000)
+        // So we must get the file id using a PROPFIND
+        // This is required so that we can detect moves even if the folder is renamed on the server
+        // while files are still uploading
+        propagator()->_activeJobList.append(this);
+        auto propfindJob = new PropfindJob(propagator()->account(), jobPath, this);
+        propfindJob->setProperties(QList<QByteArray>() << "http://owncloud.org/ns:id");
+        QObject::connect(propfindJob, &PropfindJob::result, this, &PropagateRemoteMkdir::propfindResult);
+        QObject::connect(propfindJob, &PropfindJob::finishedWithError, this, &PropagateRemoteMkdir::propfindError);
+        propfindJob->start();
+        _job = propfindJob;
+        return;
+    }
+
+    if (!_uploadEncryptedHelper && !_item->_isEncrypted) {
+        success();
+    } else {
+        // We still need to mark that folder encrypted in case we were uploading it as encrypted one
+        // Another scenario, is we are creating a new folder because of move operation on an encrypted folder that works via remove + re-upload
+        propagator()->_activeJobList.append(this);
+
+        // We're expecting directory path in /Foo/Bar convention...
+        Q_ASSERT(jobPath.startsWith('/') && !jobPath.endsWith('/'));
+        // But encryption job expect it in Foo/Bar/ convention
+        auto job = new OCC::EncryptFolderJob(propagator()->account(), propagator()->_journal, jobPath.mid(1), _item->_fileId, this);
+        connect(job, &OCC::EncryptFolderJob::finished, this, &PropagateRemoteMkdir::slotEncryptFolderFinished);
+        job->start();
+    }
+}
+
 void PropagateRemoteMkdir::slotMkdir()
 {
-    const auto rootPath = [=]() {
-        const auto result = propagator()->_remoteFolder;
-        if (result.startsWith('/')) {
-            return result.mid(1);
-        } else {
-            return result;
-        }
-    }();
     const auto path = _item->_file;
     const auto slashPosition = path.lastIndexOf('/');
     const auto parentPath = slashPosition >= 0 ? path.left(slashPosition) : QString();
@@ -163,21 +182,14 @@ void PropagateRemoteMkdir::slotMkdir()
         return;
     }
 
-    const auto remoteParentPath = parentRec._e2eMangledName.isEmpty() ? parentPath : parentRec._e2eMangledName;
-    const auto absoluteRemoteParentPath = remoteParentPath.isEmpty() ? rootPath : rootPath + remoteParentPath + '/';
-    const auto account = propagator()->account();
-
-    if (!account->capabilities().clientSideEncryptionAvailable() ||
-        (!account->e2e()->isFolderEncrypted(absoluteRemoteParentPath) &&
-         !account->e2e()->isAnyParentFolderEncrypted(absoluteRemoteParentPath))) {
+    if (!hasEncryptedAncestor()) {
         slotStartMkcolJob();
         return;
     }
 
     // We should be encrypted as well since our parent is
+    const auto remoteParentPath = parentRec._e2eMangledName.isEmpty() ? parentPath : parentRec._e2eMangledName;
     _uploadEncryptedHelper = new PropagateUploadEncrypted(propagator(), remoteParentPath, _item, this);
-    connect(_uploadEncryptedHelper, &PropagateUploadEncrypted::folderNotEncrypted,
-      this, &PropagateRemoteMkdir::slotStartMkcolJob);
     connect(_uploadEncryptedHelper, &PropagateUploadEncrypted::finalized,
       this, &PropagateRemoteMkdir::slotStartEncryptedMkcolJob);
     connect(_uploadEncryptedHelper, &PropagateUploadEncrypted::error,
@@ -193,60 +205,25 @@ void PropagateRemoteMkdir::slotMkcolJobFinished()
 
     QNetworkReply::NetworkError err = _job->reply()->error();
     _item->_httpErrorCode = _job->reply()->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt();
-
-    if (_item->_httpErrorCode == 405) {
-        // This happens when the directory already exists. Nothing to do.
-    } else if (err != QNetworkReply::NoError) {
-        SyncFileItem::Status status = classifyError(err, _item->_httpErrorCode,
-            &propagator()->_anotherSyncNeeded);
-        done(status, _job->errorString());
-        return;
-    } else if (_item->_httpErrorCode != 201) {
-        // Normally we expect "201 Created"
-        // If it is not the case, it might be because of a proxy or gateway intercepting the request, so we must
-        // throw an error.
-        done(SyncFileItem::NormalError,
-            tr("Wrong HTTP code returned by server. Expected 201, but received \"%1 %2\".")
-                .arg(_item->_httpErrorCode)
-                .arg(_job->reply()->attribute(QNetworkRequest::HttpReasonPhraseAttribute).toString()));
-        return;
-    }
-
     _item->_responseTimeStamp = _job->responseTimestamp();
+    _item->_requestId = _job->requestId();
+
     _item->_fileId = _job->reply()->rawHeader("OC-FileId");
 
-    if (_item->_fileId.isEmpty()) {
-        // Owncloud 7.0.0 and before did not have a header with the file id.
-        // (https://github.com/owncloud/core/issues/9000)
-        // So we must get the file id using a PROPFIND
-        // This is required so that we can detect moves even if the folder is renamed on the server
-        // while files are still uploading
-        propagator()->_activeJobList.append(this);
-        auto propfindJob = new PropfindJob(_job->account(), _job->path(), this);
-        propfindJob->setProperties(QList<QByteArray>() << "getetag"
-                                                       << "http://owncloud.org/ns:id");
-        QObject::connect(propfindJob, &PropfindJob::result, this, &PropagateRemoteMkdir::propfindResult);
-        QObject::connect(propfindJob, &PropfindJob::finishedWithError, this, &PropagateRemoteMkdir::propfindError);
-        propfindJob->start();
-        _job = propfindJob;
-        return;
-    }
+    _item->_errorString = _job->errorString();
 
-    if (!_uploadEncryptedHelper) {
-        success();
+    const auto jobHttpReasonPhraseString = _job->reply()->attribute(QNetworkRequest::HttpReasonPhraseAttribute).toString();
+
+    const auto jobPath = _job->path();
+
+    if (_uploadEncryptedHelper && _uploadEncryptedHelper->isFolderLocked() && !_uploadEncryptedHelper->isUnlockRunning()) {
+        // since we are done, we need to unlock a folder in case it was locked
+        connect(_uploadEncryptedHelper, &PropagateUploadEncrypted::folderUnlocked, this, [this, err, jobHttpReasonPhraseString, jobPath]() {
+            finalizeMkColJob(err, jobHttpReasonPhraseString, jobPath);
+        });
+        _uploadEncryptedHelper->unlockFolder();
     } else {
-        // We still need to mark that folder encrypted
-        propagator()->_activeJobList.append(this);
-
-        // We're expecting directory path in /Foo/Bar convention...
-        Q_ASSERT(_job->path().startsWith('/') && !_job->path().endsWith('/'));
-        // But encryption job expect it in Foo/Bar/ convention
-        // (otherwise we won't store the right string in the e2e object)
-        const auto path = QString(_job->path().mid(1) + '/');
-
-        auto job = new OCC::EncryptFolderJob(propagator()->account(), path, _item->_fileId, this);
-        connect(job, &OCC::EncryptFolderJob::finished, this, &PropagateRemoteMkdir::slotEncryptFolderFinished);
-        job->start();
+        finalizeMkColJob(err, jobHttpReasonPhraseString, jobPath);
     }
 }
 
@@ -254,15 +231,13 @@ void PropagateRemoteMkdir::slotEncryptFolderFinished()
 {
     qCDebug(lcPropagateRemoteMkdir) << "Success making the new folder encrypted";
     propagator()->_activeJobList.removeOne(this);
+    _item->_isEncrypted = true;
     success();
 }
 
 void PropagateRemoteMkdir::propfindResult(const QVariantMap &result)
 {
     propagator()->_activeJobList.removeOne(this);
-    if (result.contains("getetag")) {
-        _item->_etag = result["getetag"].toByteArray();
-    }
     if (result.contains("id")) {
         _item->_fileId = result["id"].toByteArray();
     }
@@ -278,9 +253,13 @@ void PropagateRemoteMkdir::propfindError()
 
 void PropagateRemoteMkdir::success()
 {
+    // Never save the etag on first mkdir.
+    // Only fully propagated directories should have the etag set.
+    auto itemCopy = *_item;
+    itemCopy._etag.clear();
+
     // save the file id already so we can detect rename or remove
-    SyncJournalFileRecord record = _item->toSyncJournalFileRecordWithInode(propagator()->_localDir + _item->destination());
-    if (!propagator()->_journal->setFileRecord(record)) {
+    if (!propagator()->updateMetadata(itemCopy)) {
         done(SyncFileItem::FatalError, tr("Error writing metadata to the database"));
         return;
     }
