@@ -25,7 +25,7 @@
 #include "propagatorjobs.h"
 #include "common/checksums.h"
 #include "syncengine.h"
-#include "propagateremotedelete.h"
+#include "deletejob.h"
 #include "common/asserts.h"
 #include "networkjobs.h"
 #include "clientsideencryption.h"
@@ -46,6 +46,8 @@ namespace OCC {
 Q_LOGGING_CATEGORY(lcPutJob, "nextcloud.sync.networkjob.put", QtInfoMsg)
 Q_LOGGING_CATEGORY(lcPollJob, "nextcloud.sync.networkjob.poll", QtInfoMsg)
 Q_LOGGING_CATEGORY(lcPropagateUpload, "nextcloud.sync.propagator.upload", QtInfoMsg)
+Q_LOGGING_CATEGORY(lcPropagateUploadV1, "nextcloud.sync.propagator.upload.v1", QtInfoMsg)
+Q_LOGGING_CATEGORY(lcPropagateUploadNG, "nextcloud.sync.propagator.upload.ng", QtInfoMsg)
 
 /**
  * We do not want to upload files that are currently being modified.
@@ -61,7 +63,7 @@ static bool fileIsStillChanging(const SyncFileItem &item)
     const QDateTime modtime = Utility::qDateTimeFromTime_t(item._modtime);
     const qint64 msSinceMod = modtime.msecsTo(QDateTime::currentDateTimeUtc());
 
-    return msSinceMod < SyncEngine::minimumFileAgeForUpload
+    return std::chrono::milliseconds(msSinceMod) < SyncEngine::minimumFileAgeForUpload
         // if the mtime is too much in the future we *do* upload the file
         && msSinceMod > -10000;
 }
@@ -97,6 +99,19 @@ void PUTFileJob::start()
     AbstractNetworkJob::start();
 }
 
+bool PUTFileJob::finished()
+{
+    _device->close();
+
+    qCInfo(lcPutJob) << "PUT of" << reply()->request().url().toString() << "FINISHED WITH STATUS"
+                     << replyStatusString()
+                     << reply()->attribute(QNetworkRequest::HttpStatusCodeAttribute)
+                     << reply()->attribute(QNetworkRequest::HttpReasonPhraseAttribute);
+
+    emit finishedSignal();
+    return true;
+}
+
 void PollJob::start()
 {
     setTimeout(120 * 1000);
@@ -104,7 +119,7 @@ void PollJob::start()
     QUrl finalUrl = QUrl::fromUserInput(accountUrl.scheme() + QLatin1String("://") + accountUrl.authority()
         + (path().startsWith('/') ? QLatin1String("") : QLatin1String("/")) + path());
     sendRequest("GET", finalUrl);
-    connect(reply(), &QNetworkReply::downloadProgress, this, &AbstractNetworkJob::resetTimeout);
+    connect(reply(), &QNetworkReply::downloadProgress, this, &AbstractNetworkJob::resetTimeout, Qt::UniqueConnection);
     AbstractNetworkJob::start();
 }
 
@@ -113,6 +128,7 @@ bool PollJob::finished()
     QNetworkReply::NetworkError err = reply()->error();
     if (err != QNetworkReply::NoError) {
         _item->_httpErrorCode = reply()->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt();
+        _item->_requestId = requestId();
         _item->_status = classifyError(err, _item->_httpErrorCode);
         _item->_errorString = errorString();
 
@@ -128,14 +144,14 @@ bool PollJob::finished()
             emit finishedSignal();
             return true;
         }
-        start();
+        QTimer::singleShot(8 * 1000, this, &PollJob::start);
         return false;
     }
 
     QByteArray jsonData = reply()->readAll().trimmed();
-    qCInfo(lcPollJob) << ">" << jsonData << "<" << reply()->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt();
     QJsonParseError jsonParseError;
-    QJsonObject status = QJsonDocument::fromJson(jsonData, &jsonParseError).object();
+    QJsonObject json = QJsonDocument::fromJson(jsonData, &jsonParseError).object();
+    qCInfo(lcPollJob) << ">" << jsonData << "<" << reply()->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt() << json << jsonParseError.errorString();
     if (jsonParseError.error != QJsonParseError::NoError) {
         _item->_errorString = tr("Invalid JSON reply from the poll URL");
         _item->_status = SyncFileItem::NormalError;
@@ -143,16 +159,23 @@ bool PollJob::finished()
         return true;
     }
 
-    if (status["unfinished"].toBool()) {
-        start();
+    auto status = json["status"].toString();
+    if (status == QLatin1String("init") || status == QLatin1String("started")) {
+        QTimer::singleShot(5 * 1000, this, &PollJob::start);
         return false;
     }
 
-    _item->_errorString = status["error"].toString();
-    _item->_status = _item->_errorString.isEmpty() ? SyncFileItem::Success : SyncFileItem::NormalError;
-    _item->_fileId = status["fileid"].toString().toUtf8();
-    _item->_etag = status["etag"].toString().toUtf8();
     _item->_responseTimeStamp = responseTimestamp();
+    _item->_httpErrorCode = json["errorCode"].toInt();
+
+    if (status == QLatin1String("finished")) {
+        _item->_status = SyncFileItem::Success;
+        _item->_fileId = json["fileId"].toString().toUtf8();
+        _item->_etag = parseEtag(json["ETag"].toString().toUtf8());
+    } else { // error
+        _item->_status = classifyError(QNetworkReply::UnknownContentError, _item->_httpErrorCode);
+        _item->_errorString = json["errorMessage"].toString();
+    }
 
     SyncJournalDb::PollInfo info;
     info._file = _item->_file;
@@ -168,18 +191,10 @@ PropagateUploadFileCommon::PropagateUploadFileCommon(OwncloudPropagator *propaga
     : PropagateItemJob(propagator, item)
     , _finished(false)
     , _deleteExisting(false)
-    , _parallelism(FullParallelism)
+    , _aborting(false)
     , _uploadEncryptedHelper(nullptr)
     , _uploadingEncrypted(false)
 {
-    const auto rootPath = [=]() {
-        const auto result = propagator->_remoteFolder;
-        if (result.startsWith('/')) {
-            return result.mid(1);
-        } else {
-            return result;
-        }
-    }();
     const auto path = _item->_file;
     const auto slashPosition = path.lastIndexOf('/');
     const auto parentPath = slashPosition >= 0 ? path.left(slashPosition) : QString();
@@ -187,23 +202,8 @@ PropagateUploadFileCommon::PropagateUploadFileCommon(OwncloudPropagator *propaga
     SyncJournalFileRecord parentRec;
     bool ok = propagator->_journal->getFileRecord(parentPath, &parentRec);
     if (!ok) {
-        done(SyncFileItem::NormalError);
         return;
     }
-
-    const auto remoteParentPath = parentRec._e2eMangledName.isEmpty() ? parentPath : parentRec._e2eMangledName;
-    const auto absoluteRemoteParentPath = remoteParentPath.isEmpty() ? rootPath : rootPath + remoteParentPath + '/';
-    const auto account = propagator->account();
-
-    if (account->capabilities().clientSideEncryptionAvailable() &&
-        account->e2e()->isFolderEncrypted(absoluteRemoteParentPath)) {
-        _parallelism = WaitForFinished;
-    }
-}
-
-PropagatorJob::JobParallelism PropagateUploadFileCommon::parallelism()
-{
-    return _parallelism;
 }
 
 void PropagateUploadFileCommon::setDeleteExisting(bool enabled)
@@ -213,14 +213,6 @@ void PropagateUploadFileCommon::setDeleteExisting(bool enabled)
 
 void PropagateUploadFileCommon::start()
 {
-    const auto rootPath = [=]() {
-        const auto result = propagator()->_remoteFolder;
-        if (result.startsWith('/')) {
-            return result.mid(1);
-        } else {
-            return result;
-        }
-    }();
     const auto path = _item->_file;
     const auto slashPosition = path.lastIndexOf('/');
     const auto parentPath = slashPosition >= 0 ? path.left(slashPosition) : QString();
@@ -232,23 +224,23 @@ void PropagateUploadFileCommon::start()
         return;
     }
 
-    const auto remoteParentPath = parentRec._e2eMangledName.isEmpty() ? parentPath : parentRec._e2eMangledName;
-    const auto absoluteRemoteParentPath = remoteParentPath.isEmpty() ? rootPath : rootPath + remoteParentPath + '/';
     const auto account = propagator()->account();
 
     if (!account->capabilities().clientSideEncryptionAvailable() ||
-        !account->e2e()->isFolderEncrypted(absoluteRemoteParentPath)) {
+        !parentRec.isValid() ||
+        !parentRec._isE2eEncrypted) {
         setupUnencryptedFile();
         return;
     }
 
+    const auto remoteParentPath = parentRec._e2eMangledName.isEmpty() ? parentPath : parentRec._e2eMangledName;
     _uploadEncryptedHelper = new PropagateUploadEncrypted(propagator(), remoteParentPath, _item, this);
-    connect(_uploadEncryptedHelper, &PropagateUploadEncrypted::folderNotEncrypted,
-            this, &PropagateUploadFileCommon::setupUnencryptedFile);
     connect(_uploadEncryptedHelper, &PropagateUploadEncrypted::finalized,
             this, &PropagateUploadFileCommon::setupEncryptedFile);
-    connect(_uploadEncryptedHelper, &PropagateUploadEncrypted::error,
-            []{ qCDebug(lcPropagateUpload) << "Error setting up encryption."; });
+    connect(_uploadEncryptedHelper, &PropagateUploadEncrypted::error, [this] {
+        qCDebug(lcPropagateUpload) << "Error setting up encryption.";
+        done(SyncFileItem::FatalError, tr("Failed to upload encrypted file."));
+    });
     _uploadEncryptedHelper->start();
 }
 
@@ -267,12 +259,12 @@ void PropagateUploadFileCommon::setupUnencryptedFile()
     _uploadingEncrypted = false;
     _fileToUpload._file = _item->_file;
     _fileToUpload._size = _item->_size;
-    _fileToUpload._path = propagator()->getFilePath(_fileToUpload._file);
+    _fileToUpload._path = propagator()->fullLocalPath(_fileToUpload._file);
     startUploadFile();
 }
 
 void PropagateUploadFileCommon::startUploadFile() {
-    if (propagator()->_abortRequested.fetchAndAddRelaxed(0)) {
+    if (propagator()->_abortRequested) {
         return;
     }
 
@@ -283,8 +275,8 @@ void PropagateUploadFileCommon::startUploadFile() {
     }
 
     // Check if we believe that the upload will fail due to remote quota limits
-    const quint64 quotaGuess = propagator()->_folderQuota.value(
-        QFileInfo(_fileToUpload._file).path(), std::numeric_limits<quint64>::max());
+    const qint64 quotaGuess = propagator()->_folderQuota.value(
+        QFileInfo(_fileToUpload._file).path(), std::numeric_limits<qint64>::max());
     if (_fileToUpload._size > quotaGuess) {
         // Necessary for blacklisting logic
         _item->_httpErrorCode = 507;
@@ -302,7 +294,7 @@ void PropagateUploadFileCommon::startUploadFile() {
 
     qDebug() << "Deleting the current";
     auto job = new DeleteJob(propagator()->account(),
-        propagator()->_remoteFolder + _fileToUpload._file,
+        propagator()->fullRemotePath(_fileToUpload._file),
         this);
     _jobs.append(job);
     connect(job, &DeleteJob::finishedSignal, this, &PropagateUploadFileCommon::slotComputeContentChecksum);
@@ -314,11 +306,11 @@ void PropagateUploadFileCommon::slotComputeContentChecksum()
 {
     qDebug() << "Trying to compute the checksum of the file";
     qDebug() << "Still trying to understand if this is the local file or the uploaded one";
-    if (propagator()->_abortRequested.fetchAndAddRelaxed(0)) {
+    if (propagator()->_abortRequested) {
         return;
     }
 
-    const QString filePath = propagator()->getFilePath(_item->_file);
+    const QString filePath = propagator()->fullLocalPath(_item->_file);
 
     // remember the modtime before checksumming to be able to detect a file
     // change during the checksum calculation - This goes inside of the _item->_file
@@ -326,7 +318,7 @@ void PropagateUploadFileCommon::slotComputeContentChecksum()
     // probably temporary one.
     _item->_modtime = FileSystem::getModTime(filePath);
 
-    QByteArray checksumType = contentChecksumType();
+    const QByteArray checksumType = propagator()->account()->capabilities().preferredUploadChecksumType();
 
     // Maybe the discovery already computed the checksum?
     // Should I compute the checksum of the original (_item->_file)
@@ -347,7 +339,7 @@ void PropagateUploadFileCommon::slotComputeContentChecksum()
         this, &PropagateUploadFileCommon::slotComputeTransmissionChecksum);
     connect(computeChecksum, &ComputeChecksum::done,
         computeChecksum, &QObject::deleteLater);
-    computeChecksum->start(filePath);
+    computeChecksum->start(_fileToUpload._path);
 }
 
 void PropagateUploadFileCommon::slotComputeTransmissionChecksum(const QByteArray &contentChecksumType, const QByteArray &contentChecksum)
@@ -374,8 +366,7 @@ void PropagateUploadFileCommon::slotComputeTransmissionChecksum(const QByteArray
         this, &PropagateUploadFileCommon::slotStartUpload);
     connect(computeChecksum, &ComputeChecksum::done,
         computeChecksum, &QObject::deleteLater);
-    const QString filePath = propagator()->getFilePath(_item->_file);
-    computeChecksum->start(filePath);
+    computeChecksum->start(_fileToUpload._path);
 }
 
 void PropagateUploadFileCommon::slotStartUpload(const QByteArray &transmissionChecksumType, const QByteArray &transmissionChecksum)
@@ -392,14 +383,10 @@ void PropagateUploadFileCommon::slotStartUpload(const QByteArray &transmissionCh
     }
 
     const QString fullFilePath = _fileToUpload._path;
-    const QString originalFilePath = propagator()->getFilePath(_item->_file);
+    const QString originalFilePath = propagator()->fullLocalPath(_item->_file);
 
     if (!FileSystem::fileExists(fullFilePath)) {
-      if (_uploadingEncrypted) {
-        _uploadEncryptedHelper->unlockFolder();
-      }
-    done(SyncFileItem::SoftError, tr("File Removed (start upload) %1").arg(fullFilePath));
-        return;
+        return slotOnErrorStartFolderUnlock(SyncFileItem::SoftError, tr("File Removed (start upload) %1").arg(fullFilePath));
     }
     time_t prevModtime = _item->_modtime; // the _item value was set in PropagateUploadFile::start()
     // but a potential checksum calculation could have taken some time during which the file could
@@ -408,39 +395,50 @@ void PropagateUploadFileCommon::slotStartUpload(const QByteArray &transmissionCh
     _item->_modtime = FileSystem::getModTime(originalFilePath);
     if (prevModtime != _item->_modtime) {
         propagator()->_anotherSyncNeeded = true;
-        if (_uploadingEncrypted) {
-          _uploadEncryptedHelper->unlockFolder();
-        }
         qDebug() << "prevModtime" << prevModtime << "Curr" << _item->_modtime;
-        done(SyncFileItem::SoftError, tr("Local file changed during syncing. It will be resumed."));
-        return;
+        return slotOnErrorStartFolderUnlock(SyncFileItem::SoftError, tr("Local file changed during syncing. It will be resumed."));
     }
 
-    quint64 fileSize = FileSystem::getSize(fullFilePath);
-    _fileToUpload._size = fileSize;
+    _fileToUpload._size = FileSystem::getSize(fullFilePath);
+    _item->_size = FileSystem::getSize(originalFilePath);
 
     // But skip the file if the mtime is too close to 'now'!
     // That usually indicates a file that is still being changed
     // or not yet fully copied to the destination.
     if (fileIsStillChanging(*_item)) {
         propagator()->_anotherSyncNeeded = true;
-        if (_uploadingEncrypted) {
-          _uploadEncryptedHelper->unlockFolder();
-        }
-        done(SyncFileItem::SoftError, tr("Local file changed during sync."));
-        return;
+        return slotOnErrorStartFolderUnlock(SyncFileItem::SoftError, tr("Local file changed during sync."));
     }
 
     doStartUpload();
 }
 
-UploadDevice::UploadDevice(BandwidthManager *bwm)
-    : _read(0)
+void PropagateUploadFileCommon::slotFolderUnlocked(const QByteArray &folderId, int httpReturnCode)
+{
+    qDebug() << "Failed to unlock encrypted folder" << folderId;
+    if (_uploadStatus.status == SyncFileItem::NoStatus && httpReturnCode != 200) {
+        done(SyncFileItem::FatalError, tr("Failed to unlock encrypted folder."));
+    } else {
+        done(_uploadStatus.status, _uploadStatus.message);
+    }
+}
+
+void PropagateUploadFileCommon::slotOnErrorStartFolderUnlock(SyncFileItem::Status status, const QString &errorString)
+{
+    if (_uploadingEncrypted) {
+        _uploadStatus = { status, errorString };
+        connect(_uploadEncryptedHelper, &PropagateUploadEncrypted::folderUnlocked, this, &PropagateUploadFileCommon::slotFolderUnlocked);
+        _uploadEncryptedHelper->unlockFolder();
+    } else {
+        done(status, errorString);
+    }
+}
+
+UploadDevice::UploadDevice(const QString &fileName, qint64 start, qint64 size, BandwidthManager *bwm)
+    : _file(fileName)
+    , _start(start)
+    , _size(size)
     , _bandwidthManager(bwm)
-    , _bandwidthQuota(0)
-    , _readWithProgress(0)
-    , _bandwidthLimited(false)
-    , _choked(false)
 {
     _bandwidthManager->registerUploadDevice(this);
 }
@@ -453,29 +451,32 @@ UploadDevice::~UploadDevice()
     }
 }
 
-bool UploadDevice::prepareAndOpen(const QString &fileName, qint64 start, qint64 size)
+bool UploadDevice::open(QIODevice::OpenMode mode)
 {
-    _data.clear();
-    _read = 0;
+    if (mode & QIODevice::WriteOnly)
+        return false;
+
+    // Get the file size now: _file.fileName() is no longer reliable
+    // on all platforms after openAndSeekFileSharedRead().
+    auto fileDiskSize = FileSystem::getSize(_file.fileName());
 
-    QFile file(fileName);
     QString openError;
-    if (!FileSystem::openAndSeekFileSharedRead(&file, &openError, start)) {
+    if (!FileSystem::openAndSeekFileSharedRead(&_file, &openError, _start)) {
         setErrorString(openError);
         return false;
     }
 
-    size = qBound(0ll, size, FileSystem::getSize(fileName) - start);
-    _data.resize(size);
-    auto read = file.read(_data.data(), size);
-    if (read != size) {
-        setErrorString(file.errorString());
-        return false;
-    }
+    _size = qBound(0ll, _size, fileDiskSize - _start);
+    _read = 0;
 
-    return QIODevice::open(QIODevice::ReadOnly);
+    return QIODevice::open(mode);
 }
 
+void UploadDevice::close()
+{
+    _file.close();
+    QIODevice::close();
+}
 
 qint64 UploadDevice::writeData(const char *, qint64)
 {
@@ -485,15 +486,15 @@ qint64 UploadDevice::writeData(const char *, qint64)
 
 qint64 UploadDevice::readData(char *data, qint64 maxlen)
 {
-    if (_data.size() - _read <= 0) {
+    if (_size - _read <= 0) {
         // at end
         if (_bandwidthManager) {
             _bandwidthManager->unregisterUploadDevice(this);
         }
         return -1;
     }
-    maxlen = qMin(maxlen, _data.size() - _read);
-    if (maxlen == 0) {
+    maxlen = qMin(maxlen, _size - _read);
+    if (maxlen <= 0) {
         return 0;
     }
     if (isChoked()) {
@@ -506,9 +507,14 @@ qint64 UploadDevice::readData(char *data, qint64 maxlen)
         }
         _bandwidthQuota -= maxlen;
     }
-    std::memcpy(data, _data.data() + _read, maxlen);
-    _read += maxlen;
-    return maxlen;
+
+    auto c = _file.read(data, maxlen);
+    if (c < 0) {
+        setErrorString(_file.errorString());
+        return -1;
+    }
+    _read += c;
+    return c;
 }
 
 void UploadDevice::slotJobUploadProgress(qint64 sent, qint64 t)
@@ -521,17 +527,17 @@ void UploadDevice::slotJobUploadProgress(qint64 sent, qint64 t)
 
 bool UploadDevice::atEnd() const
 {
-    return _read >= _data.size();
+    return _read >= _size;
 }
 
 qint64 UploadDevice::size() const
 {
-    return _data.size();
+    return _size;
 }
 
 qint64 UploadDevice::bytesAvailable() const
 {
-    return _data.size() - _read + QIODevice::bytesAvailable();
+    return _size - _read + QIODevice::bytesAvailable();
 }
 
 // random access, we can seek
@@ -545,10 +551,11 @@ bool UploadDevice::seek(qint64 pos)
     if (!QIODevice::seek(pos)) {
         return false;
     }
-    if (pos < 0 || pos > _data.size()) {
+    if (pos < 0 || pos > _size) {
         return false;
     }
     _read = pos;
+    _file.seek(_start + pos);
     return true;
 }
 
@@ -577,12 +584,13 @@ void UploadDevice::setChoked(bool b)
 void PropagateUploadFileCommon::startPollJob(const QString &path)
 {
     auto *job = new PollJob(propagator()->account(), path, _item,
-        propagator()->_journal, propagator()->_localDir, this);
+        propagator()->_journal, propagator()->localPath(), this);
     connect(job, &PollJob::finishedSignal, this, &PropagateUploadFileCommon::slotPollFinished);
     SyncJournalDb::PollInfo info;
     info._file = _item->_file;
     info._url = path;
     info._modtime = _item->_modtime;
+    info._fileSize = _item->_size;
     propagator()->_journal->setPollInfo(info);
     propagator()->_journal->commit("add poll info");
     propagator()->_activeJobList.append(this);
@@ -641,7 +649,7 @@ void PropagateUploadFileCommon::commonErrorHandling(AbstractNetworkJob *job)
 
         // Maybe the bad etag is in the database, we need to clear the
         // parent folder etag so we won't read from DB next sync.
-        propagator()->_journal->avoidReadFromDbOnNextSync(_item->_file);
+        propagator()->_journal->schedulePathForRemoteDiscovery(_item->_file);
         propagator()->_anotherSyncNeeded = true;
     }
 
@@ -649,7 +657,7 @@ void PropagateUploadFileCommon::commonErrorHandling(AbstractNetworkJob *job)
     checkResettingErrors();
 
     SyncFileItem::Status status = classifyError(job->reply()->error(), _item->_httpErrorCode,
-        &propagator()->_anotherSyncNeeded);
+        &propagator()->_anotherSyncNeeded, replyContent);
 
     // Insufficient remote storage.
     if (_item->_httpErrorCode == 507) {
@@ -674,7 +682,7 @@ void PropagateUploadFileCommon::commonErrorHandling(AbstractNetworkJob *job)
     abortWithError(status, errorString);
 }
 
-void PropagateUploadFileCommon::adjustLastJobTimeout(AbstractNetworkJob *job, quint64 fileSize)
+void PropagateUploadFileCommon::adjustLastJobTimeout(AbstractNetworkJob *job, qint64 fileSize)
 {
     constexpr double threeMinutes = 3.0 * 60 * 1000;
 
@@ -694,6 +702,8 @@ void PropagateUploadFileCommon::slotJobDestroyed(QObject *job)
 // This function is used whenever there is an error occuring and jobs might be in progress
 void PropagateUploadFileCommon::abortWithError(SyncFileItem::Status status, const QString &error)
 {
+    if (_aborting)
+        return;
     abort(AbortType::Synchronous);
     done(status, error);
 }
@@ -701,11 +711,12 @@ void PropagateUploadFileCommon::abortWithError(SyncFileItem::Status status, cons
 QMap<QByteArray, QByteArray> PropagateUploadFileCommon::headers()
 {
     QMap<QByteArray, QByteArray> headers;
-    headers["OC-Async"] = "1";
-    headers["Content-Type"] = "application/octet-stream";
-    headers["X-OC-Mtime"] = QByteArray::number(qint64(_item->_modtime));
+    headers[QByteArrayLiteral("Content-Type")] = QByteArrayLiteral("application/octet-stream");
+    headers[QByteArrayLiteral("X-OC-Mtime")] = QByteArray::number(qint64(_item->_modtime));
+    if (qEnvironmentVariableIntValue("OWNCLOUD_LAZYOPS"))
+        headers[QByteArrayLiteral("OC-LazyOps")] = QByteArrayLiteral("true");
 
-    if (_item->_file.contains(".sys.admin#recall#")) {
+    if (_item->_file.contains(QLatin1String(".sys.admin#recall#"))) {
         // This is a file recall triggered by the admin.  Note: the
         // recall list file created by the admin and downloaded by the
         // client (.sys.admin#recall#) also falls into this category
@@ -722,23 +733,25 @@ QMap<QByteArray, QByteArray> PropagateUploadFileCommon::headers()
         && !_deleteExisting) {
         // We add quotes because the owncloud server always adds quotes around the etag, and
         //  csync_owncloud.c's owncloud_file_id always strips the quotes.
-        headers["If-Match"] = '"' + _item->_etag + '"';
+        headers[QByteArrayLiteral("If-Match")] = '"' + _item->_etag + '"';
     }
 
     // Set up a conflict file header pointing to the original file
     auto conflictRecord = propagator()->_journal->conflictRecord(_item->_file.toUtf8());
     if (conflictRecord.isValid()) {
-        headers["OC-Conflict"] = "1";
+        headers[QByteArrayLiteral("OC-Conflict")] = "1";
+        if (!conflictRecord.initialBasePath.isEmpty())
+            headers[QByteArrayLiteral("OC-ConflictInitialBasePath")] = conflictRecord.initialBasePath;
         if (!conflictRecord.baseFileId.isEmpty())
-            headers["OC-ConflictBaseFileId"] = conflictRecord.baseFileId;
+            headers[QByteArrayLiteral("OC-ConflictBaseFileId")] = conflictRecord.baseFileId;
         if (conflictRecord.baseModtime != -1)
-            headers["OC-ConflictBaseMtime"] = QByteArray::number(conflictRecord.baseModtime);
+            headers[QByteArrayLiteral("OC-ConflictBaseMtime")] = QByteArray::number(conflictRecord.baseModtime);
         if (!conflictRecord.baseEtag.isEmpty())
-            headers["OC-ConflictBaseEtag"] = conflictRecord.baseEtag;
+            headers[QByteArrayLiteral("OC-ConflictBaseEtag")] = conflictRecord.baseEtag;
     }
 
-    if (_uploadEncryptedHelper && !_uploadEncryptedHelper->_folderToken.isEmpty()) {
-        headers.insert("e2e-token", _uploadEncryptedHelper->_folderToken);
+    if (_uploadEncryptedHelper && !_uploadEncryptedHelper->folderToken().isEmpty()) {
+        headers.insert("e2e-token", _uploadEncryptedHelper->folderToken());
     }
 
     return headers;
@@ -751,28 +764,44 @@ void PropagateUploadFileCommon::finalize()
     if (quotaIt != propagator()->_folderQuota.end())
         quotaIt.value() -= _fileToUpload._size;
 
-    // Update the database entry - use the local file, not the temporary one.
-    const auto filePath = propagator()->getFilePath(_item->_file);
-    const auto fileRecord = _item->toSyncJournalFileRecordWithInode(filePath);
-    if (!propagator()->_journal->setFileRecord(fileRecord)) {
+    // Update the database entry
+    if (!propagator()->updateMetadata(*_item)) {
         done(SyncFileItem::FatalError, tr("Error writing metadata to the database"));
         return;
     }
 
+    // Files that were new on the remote shouldn't have online-only pin state
+    // even if their parent folder is online-only.
+    if (_item->_instruction == CSYNC_INSTRUCTION_NEW
+        || _item->_instruction == CSYNC_INSTRUCTION_TYPE_CHANGE) {
+        auto &vfs = propagator()->syncOptions()._vfs;
+        const auto pin = vfs->pinState(_item->_file);
+        if (pin && *pin == PinState::OnlineOnly) {
+            vfs->setPinState(_item->_file, PinState::Unspecified);
+        }
+    }
+
     // Remove from the progress database:
     propagator()->_journal->setUploadInfo(_item->_file, SyncJournalDb::UploadInfo());
     propagator()->_journal->commit("upload file start");
 
     if (_uploadingEncrypted) {
-      _uploadEncryptedHelper->unlockFolder();
+        _uploadStatus = { SyncFileItem::Success, QString() };
+        connect(_uploadEncryptedHelper, &PropagateUploadEncrypted::folderUnlocked, this, &PropagateUploadFileCommon::slotFolderUnlocked);
+        _uploadEncryptedHelper->unlockFolder();
+    } else {
+        done(SyncFileItem::Success);
     }
-    done(SyncFileItem::Success);
 }
 
 void PropagateUploadFileCommon::abortNetworkJobs(
     PropagatorJob::AbortType abortType,
     const std::function<bool(AbstractNetworkJob *)> &mayAbortJob)
 {
+    if (_aborting)
+        return;
+    _aborting = true;
+
     // Count the number of jobs that need aborting, and emit the overall
     // abort signal when they're all done.
     QSharedPointer<int> runningCount(new int(0));
