@@ -33,17 +33,17 @@ class PropagateUploadEncrypted : public QObject
   Q_OBJECT
 public:
     PropagateUploadEncrypted(OwncloudPropagator *propagator, const QString &remoteParentPath, SyncFileItemPtr item, QObject *parent = nullptr);
+    ~PropagateUploadEncrypted() = default;
+
     void start();
 
-    /* unlocks the current folder that holds this file */
     void unlockFolder();
-  // Used by propagateupload
-  QByteArray _folderToken;
-  QByteArray _folderId;
+
+    bool isUnlockRunning() const { return _isUnlockRunning; }
+    bool isFolderLocked() const { return _isFolderLocked; }
+    const QByteArray folderToken() const { return _folderToken; }
 
 private slots:
-    void slotFolderEncryptedStatusFetched(const QString &folder, bool isEncrypted);
-    void slotFolderEncryptedStatusError(int error);
     void slotFolderEncryptedIdReceived(const QStringList &list);
     void slotFolderEncryptedIdError(QNetworkReply *r);
     void slotFolderLockedSuccessfully(const QByteArray& fileId, const QByteArray& token);
@@ -58,18 +58,22 @@ signals:
     // Emmited after the file is encrypted and everythign is setup.
     void finalized(const QString& path, const QString& filename, quint64 size);
     void error();
-
-    // Emited if the file is not in a encrypted folder.
-    void folderNotEncrypted();
+    void folderUnlocked(const QByteArray &folderId, int httpStatus);
 
 private:
   OwncloudPropagator *_propagator;
   QString _remoteParentPath;
   SyncFileItemPtr _item;
 
+  QByteArray _folderToken;
+  QByteArray _folderId;
+
   QElapsedTimer _folderLockFirstTry;
   bool _currentLockingInProgress = false;
 
+  bool _isUnlockRunning = false;
+  bool _isFolderLocked = false;
+
   QByteArray _generatedKey;
   QByteArray _generatedIv;
   FolderMetadata *_metadata;
