@@ -16,6 +16,8 @@
 #include "config.h"
 #include "common/utility.h"
 #include "version.h"
+#include "configfile.h"
+#include "common/vfs.h"
 
 #include <QtCore>
 #ifndef TOKEN_AUTH_ONLY
@@ -50,6 +52,11 @@ QUrl imagePathToUrl(const QString &imagePath)
     }
 }
 
+bool shouldPreferSvg()
+{
+    return QByteArray(APPLICATION_ICON_SET).toUpper() == QByteArrayLiteral("SVG");
+}
+
 }
 
 namespace OCC {
@@ -132,6 +139,26 @@ QUrl Theme::stateOfflineImageSource() const
     return imagePathToUrl(themeImagePath("state-offline", 16));
 }
 
+QUrl Theme::statusOnlineImageSource() const
+{
+    return imagePathToUrl(themeImagePath("user-status-online", 16));
+}
+
+QUrl Theme::statusDoNotDisturbImageSource() const
+{
+    return imagePathToUrl(themeImagePath("user-status-dnd", 16));
+}
+
+QUrl Theme::statusAwayImageSource() const
+{
+    return imagePathToUrl(themeImagePath("user-status-away", 16));
+}
+
+QUrl Theme::statusInvisibleImageSource() const
+{
+    return imagePathToUrl(themeImagePath("user-status-invisible", 16));
+}
+
 QString Theme::version() const
 {
     return MIRALL_VERSION_STRING;
@@ -185,10 +212,11 @@ QIcon Theme::themeIcon(const QString &name, bool sysTray) const
             return QPixmap(pixmapName);
         };
 
-        const auto sizes = isBranded() ? QVector<int>{ 16, 22, 32, 48, 64, 128, 256, 512, 1024 }
-                                       : QVector<int>{ 16, 32, 64, 128, 256 };
+        const auto useSvg = shouldPreferSvg();
+        const auto sizes = useSvg ? QVector<int>{ 16, 32, 64, 128, 256 }
+                                  : QVector<int>{ 16, 22, 32, 48, 64, 128, 256, 512, 1024 };
         for (int size : sizes) {
-            auto px = isBranded() ? loadPixmap(size) : createPixmapFromSvg(size);
+            auto px = useSvg ? createPixmapFromSvg(size) : loadPixmap(size);
             if (px.isNull()) {
                 continue;
             }
@@ -217,20 +245,31 @@ QIcon Theme::themeIcon(const QString &name, bool sysTray) const
 QString Theme::themeImagePath(const QString &name, int size, bool sysTray) const
 {
     const auto flavor = (!isBranded() && sysTray) ? systrayIconFlavor(_mono) : QLatin1String("colored");
+    const auto useSvg = shouldPreferSvg();
 
     // branded client may have several sizes of the same icon
-    const QString filePath = (isBranded() && size > 0)
-            ? QString::fromLatin1(":/client/theme/%1/%2-%3").arg(flavor).arg(name).arg(size)
-            : QString::fromLatin1(":/client/theme/%1/%2").arg(flavor).arg(name);
+    const QString filePath = (useSvg || size <= 0)
+            ? QString::fromLatin1(":/client/theme/%1/%2").arg(flavor).arg(name)
+            : QString::fromLatin1(":/client/theme/%1/%2-%3").arg(flavor).arg(name).arg(size);
 
-    const QString brandedImagePath = filePath + ".png";
-
-    // only return branded .png image path if it exists, or fall-back to non-branded .svg otherwise
-    if (isBranded() && QFile::exists(brandedImagePath)) {
-        return brandedImagePath;
+    const QString svgPath = filePath + ".svg";
+    if (useSvg) {
+        return svgPath;
     }
 
-    return filePath + ".svg";
+    const QString pngPath = filePath + ".png";
+    // Use the SVG as fallback if a PNG is missing so that we get a chance to display something
+    if (QFile::exists(pngPath)) {
+        return pngPath;
+    } else {
+        return svgPath;
+    }
+}
+
+bool Theme::isHidpi(QPaintDevice *dev)
+{
+    const auto devicePixelRatio = dev ? dev->devicePixelRatio() : qApp->primaryScreen()->devicePixelRatio();
+    return devicePixelRatio > 1;
 }
 
 QIcon Theme::uiThemeIcon(const QString &iconName, bool uiHasDarkBg) const
@@ -243,8 +282,7 @@ QIcon Theme::uiThemeIcon(const QString &iconName, bool uiHasDarkBg) const
 
 QString Theme::hidpiFileName(const QString &fileName, QPaintDevice *dev)
 {
-    qreal devicePixelRatio = dev ? dev->devicePixelRatio() : qApp->primaryScreen()->devicePixelRatio();
-    if (devicePixelRatio <= 1.0) {
+    if (!Theme::isHidpi(dev)) {
         return fileName;
     }
     // try to find a 2x version
@@ -261,6 +299,16 @@ QString Theme::hidpiFileName(const QString &fileName, QPaintDevice *dev)
     return fileName;
 }
 
+QString Theme::hidpiFileName(const QString &iconName, const QColor &backgroundColor, QPaintDevice *dev)
+{
+    const auto isDarkBackground = Theme::isDarkColor(backgroundColor);
+
+    const QString themeResBasePath = ":/client/theme/";
+    const QString iconPath = themeResBasePath + (isDarkBackground ? "white/" : "black/") + iconName;
+
+    return Theme::hidpiFileName(iconPath, dev);
+}
+
 
 #endif
 
@@ -418,6 +466,9 @@ QString Theme::about() const
               .arg(QString::fromLatin1(MIRALL_STRINGIFY(MIRALL_VERSION)) + QString(" (%1)").arg(osName))
               .arg(helpUrl());
 
+    devString += QString("<p><small>Using virtual files plugin: %1</small></p>")
+        .arg(Vfs::modeToString(bestAvailableVfsMode()));
+
     return devString;
 }
 
@@ -525,10 +576,43 @@ QColor Theme::wizardHeaderBackgroundColor() const
     return {APPLICATION_WIZARD_HEADER_BACKGROUND_COLOR};
 }
 
+QPixmap Theme::wizardApplicationLogo() const
+{
+    if (!Theme::isBranded()) {
+        return QPixmap(Theme::hidpiFileName(":/client/theme/colored/wizard-nextcloud.png"));
+    }
+#ifdef APPLICATION_WIZARD_USE_CUSTOM_LOGO
+    const auto useSvg = shouldPreferSvg();
+    const auto logoBasePath = QStringLiteral(":/client/theme/colored/wizard_logo");
+    if (useSvg) {
+        const auto maxHeight = Theme::isHidpi() ? 200 : 100;
+        const auto maxWidth = 2 * maxHeight;
+        const auto icon = QIcon(logoBasePath + ".svg");
+        const auto size = icon.actualSize(QSize(maxWidth, maxHeight));
+        return icon.pixmap(size);
+    } else {
+        return QPixmap(hidpiFileName(logoBasePath + ".png"));
+    }
+#else
+    const auto size = Theme::isHidpi() ?: 200 : 100;
+    return applicationIcon().pixmap(size);
+#endif
+}
+
 QPixmap Theme::wizardHeaderLogo() const
 {
 #ifdef APPLICATION_WIZARD_USE_CUSTOM_LOGO
-   return QPixmap(hidpiFileName(":/client/theme/colored/wizard_logo.png"));
+    const auto useSvg = shouldPreferSvg();
+    const auto logoBasePath = QStringLiteral(":/client/theme/colored/wizard_logo");
+    if (useSvg) {
+        const auto maxHeight = 64;
+        const auto maxWidth = 2 * maxHeight;
+        const auto icon = QIcon(logoBasePath + ".svg");
+        const auto size = icon.actualSize(QSize(maxWidth, maxHeight));
+        return icon.pixmap(size);
+    } else {
+        return QPixmap(hidpiFileName(logoBasePath + ".png"));
+    }
 #else
     return applicationIcon().pixmap(64);
 #endif
@@ -731,4 +815,10 @@ QPixmap Theme::createColorAwarePixmap(const QString &name)
     return createColorAwarePixmap(name, QGuiApplication::palette());
 }
 
+bool Theme::showVirtualFilesOption() const
+{
+    const auto vfsMode = bestAvailableVfsMode();
+    return ConfigFile().showExperimentalOptions() || vfsMode == Vfs::WindowsCfApi;
+}
+
 } // end namespace client
