@@ -9,6 +9,7 @@ void OCC::AccountManager::save(bool) { }
 OCC::AccountState *OCC::AccountManager::addAccount(const AccountPtr& ac) { return new OCC::AccountState(ac); }
 void OCC::AccountManager::deleteAccount(AccountState *) { }
 void OCC::AccountManager::accountRemoved(OCC::AccountState*) { }
+QList<OCC::AccountStatePtr> OCC::AccountManager::accounts() const { return QList<OCC::AccountStatePtr>(); }
 OCC::AccountStatePtr OCC::AccountManager::account(const QString &){ return AccountStatePtr(); }
 const QMetaObject OCC::AccountManager::staticMetaObject = QObject::staticMetaObject;
 
@@ -20,7 +21,7 @@ QString OCC::FolderMan::unescapeAlias(QString const&){ return QString(); }
 QString OCC::FolderMan::escapeAlias(QString const&){ return QString(); }
 void OCC::FolderMan::scheduleFolder(OCC::Folder*){ }
 OCC::SocketApi *OCC::FolderMan::socketApi(){ return new SocketApi;  }
-OCC::Folder::Map OCC::FolderMan::map() { return OCC::Folder::Map(); }
+const OCC::Folder::Map &OCC::FolderMan::map() const { return OCC::Folder::Map(); }
 void OCC::FolderMan::setSyncEnabled(bool) { }
 void OCC::FolderMan::slotSyncOnceFileUnlocks(QString const&) { }
 void OCC::FolderMan::slotScheduleETagJob(QString const&, OCC::RequestEtagJob*){ }
